"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StepScheduler = void 0;
const node_util_1 = require("node:util");
const utility_1 = require("@jfrog-pipelines/utility");
const metadata_1 = require("@jfrog-pipelines/metadata");
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const configuration_1 = require("@jfrog-pipelines/configuration");
const logger_1 = require("@jfrog-pipelines/logger");
const BatchStepletConsoleAdapter_js_1 = __importDefault(require("./BatchStepletConsoleAdapter.js"));
const RunDao_1 = require("./modules/RunDao");
const constants_1 = require("./constants");
const ONE_DAY_IN_SECONDS = 86400;
class StepScheduler {
    constructor(waitingGroups, totalGroupsThatCanBeQueued, nodePool, groupProjects) {
        this.waitingGroups = waitingGroups;
        this.totalGroupsThatCanBeQueued = totalGroupsThatCanBeQueued;
        this.nodePool = nodePool;
        this.groupProjects = groupProjects;
        this.config = configuration_1.ConfigFactory.config;
        this.logger = logger_1.LoggerFactory.getLogger('StepScheduler');
    }
    async queueStepsToNodePool() {
        this.logger.debug(`Total groups that can be queued: ${this.totalGroupsThatCanBeQueued}`);
        try {
            const waitingGroupsThatCanBeQueued = this.waitingGroups.slice(0, this.totalGroupsThatCanBeQueued);
            for await (const waitingGroup of waitingGroupsThatCanBeQueued) {
                await this.tryAndQueueWaitingGroup(waitingGroup);
            }
            return;
        }
        catch (error) {
            this.logger.info('Failed to queue groups', (0, node_util_1.inspect)(error));
        }
    }
    async tryAndQueueWaitingGroup(waitingGroup) {
        const startTime = Date.now();
        await this.isNodeAvailable();
        if (waitingGroup.isGroupSkipped()) {
            await this.skipWaitingGroup(waitingGroup);
            return;
        }
        const localArtifactoryToken = await this.createLocalArtifactoryToken(waitingGroup);
        const runSetupToken = await this.createRunStepToken(waitingGroup);
        const runExecutionToken = await this.checkAndAddRunExecutionToken(waitingGroup);
        await this.queueWaitingGroup(waitingGroup, runSetupToken, runExecutionToken, localArtifactoryToken);
        const stopTime = Date.now();
        this.logEvent(waitingGroup, new Date(stopTime - startTime));
    }
    async isNodeAvailable() {
        if (this.totalGroupsThatCanBeQueued <= 0) {
            throw new Error(`Node pool ${this.nodePool.name} has insufficient capacity. Skipping.`);
        }
        const pipelinesClient = new internal_clients_1.PipelinesClient(12);
        const executionNodes = await pipelinesClient.getExecutionNodes();
        if (executionNodes.available <= 0) {
            throw new Error(`Insufficient license capacity. Skipping ${this.nodePool.name}`);
        }
        return true;
    }
    async createRunStepToken(group) {
        const tokenRequest = {
            username: `run-setup-${group.runId}@jfpip`,
            scope: `internal:pipelines/project/${group.projectId}:r,w,d,m`,
            refreshable: false,
            audience: constants_1.WILDCARD_AUDIENCE,
            expiresIn: ONE_DAY_IN_SECONDS,
            grantType: 'client_credentials'
        };
        return this.createToken(tokenRequest);
    }
    async createLocalArtifactoryToken(group) {
        if (!group.isStepUsingLocalArtifactory) {
            return;
        }
        const project = (0, utility_1.find)(this.groupProjects, { id: group.projectId });
        if (!project) {
            return;
        }
        const pipelinesClient = new internal_clients_1.PipelinesClient(12);
        const pipelineSource = await pipelinesClient.pipelineSource(group.pipelineSourceId);
        const localArtifactoryIntegrationTokenPermissions = !(0, utility_1.isEmpty)(pipelineSource.artifactoryTokenPermissions)
            ? pipelineSource.artifactoryTokenPermissions.map((action) => action[0])
            : ['r'];
        const tokenActions = localArtifactoryIntegrationTokenPermissions.join(',');
        const tokenRequest = {
            username: `run-setup-${group.runId}@jfpip`,
            scope: `artifact:${project.sourceId}-*:${tokenActions}`,
            refreshable: false,
            audience: constants_1.WILDCARD_AUDIENCE,
            expiresIn: ONE_DAY_IN_SECONDS,
            grantType: 'client_credentials'
        };
        return this.createToken(tokenRequest);
    }
    async checkAndAddRunExecutionToken(group) {
        const runDao = new RunDao_1.RunDao();
        const run = await runDao.findById(group.runId);
        const systemPropertyBag = run?.systemPropertyBag ?? {};
        if (!(0, utility_1.isEmpty)(systemPropertyBag) && !(0, utility_1.isEmpty)(systemPropertyBag.token)) {
            return systemPropertyBag.token;
        }
        const tokenRequest = {
            username: `run-${group.runId}@jfpip`,
            scope: `internal:pipelines/project/${group.projectId}:r,w,d`,
            refreshable: false,
            audience: constants_1.WILDCARD_AUDIENCE,
            expiresIn: ONE_DAY_IN_SECONDS,
            grantType: 'client_credentials'
        };
        const accessToken = await this.createToken(tokenRequest);
        const updateSystemPropertyBag = { ...systemPropertyBag, token: accessToken };
        await runDao.updateByInstance(run, {
            systemPropertyBag: updateSystemPropertyBag
        });
        return accessToken;
    }
    async skipWaitingGroup(group) {
        await this.postStepSkippedConsoles(group);
        await this.skipSteps(group);
        await this.skipSteplets(group);
        return;
    }
    async postQueuedStepConsoles(group) {
        const steplets = group.steplets;
        const stepletIds = (0, utility_1.map)(steplets, 'id');
        const consoleAdapter = new BatchStepletConsoleAdapter_js_1.default((0, utility_1.uuidv4)(), this.config.apiToken, stepletIds, group.pipelineId, this.config.jobConsoleBatchSize);
        const grpMessage = 'Queuing step';
        const logMessage = `Successfully queued step to node pool ${group.nodePoolName}. Waiting for node to start execution.`;
        consoleAdapter.openGrp(grpMessage);
        consoleAdapter.openCmd('Info');
        consoleAdapter.publishMsg(logMessage);
        consoleAdapter.closeCmd(true);
        consoleAdapter.closeGrp(true);
        const flushConsolesPromisified = (0, node_util_1.promisify)(consoleAdapter.flushConsoles).bind(consoleAdapter);
        await flushConsolesPromisified();
    }
    async queueWaitingGroup(group, runSetupToken, runToken, localArtifactoryToken) {
        const stepsToExecute = group.getStepsToExecute();
        const stepIdsToExecute = (0, utility_1.map)(stepsToExecute, 'stepId');
        const providerMetadataPropertyBag = this.nodePool.providerMetadataPropertyBag;
        const instanceSize = providerMetadataPropertyBag?.instanceSize;
        const updateStep = {
            groupInProgress: true,
            groupStartedAt: new Date(),
            configPropertyBag: {
                isOnDemand: this.nodePool.isOnDemand,
                instanceSize
            },
            systemPropertyBag: {
                runSetupToken: runSetupToken
            }
        };
        const messagePayload = {
            affinityGroup: group.affinityGroup,
            runId: group.runId,
            stepIds: stepIdsToExecute,
            steps: stepsToExecute,
            builderApiToken: runToken,
            builderSetupApiToken: runSetupToken,
            localArtifactoryToken: localArtifactoryToken
        };
        const message = {
            projectId: group.projectId,
            pipelineId: group.pipelineId,
            runId: group.runId,
            payload: messagePayload,
            nodePoolId: this.nodePool.id,
            priority: group.priority,
            triggeredAt: group.triggeredAt,
            statusCode: metadata_1.READY,
            updateStep: updateStep
        };
        const pipelinesClient = new internal_clients_1.PipelinesClient(12);
        await pipelinesClient.queueToNodePool(message);
        await this.postQueuedStepConsoles(group);
        return;
    }
    logEvent(waitingGroup, timeDiff) {
        const eventData = {
            timestamp: new Date().toISOString(),
            domain: 'stepService',
            eventType: 'queueWaitingGroup',
            status: 'sucess',
            queuedStepsCount: waitingGroup.stepIds.length,
            durationMillis: timeDiff.valueOf(),
            affinityGroup: waitingGroup.affinityGroup,
            nodePoolName: waitingGroup.nodePoolName,
            pipelinesSessionId: (0, utility_1.uuidv4)()
        };
        logger_1.LoggerFactory.logEvent(eventData);
    }
    getSkippedMessage(group) {
        let grpMessage = '';
        let logMessage = '';
        if (group.skipByNoNode) {
            grpMessage = 'Step skipped';
            logMessage = 'Skipping step as no node is found. ' + 'Add a node to run a step.';
        }
        else if (group.skipByNodePoolPermissions) {
            grpMessage = 'Step skipped';
            logMessage =
                'Skipping step as it is part of a pipeline source that ' +
                    'does not have permission to use the specified nodePool.';
        }
        else if (group.skipByNonRootEnforcement) {
            grpMessage = 'Step skipped';
            logMessage =
                'Cannot initialize node pool with a root user' +
                    ' as a global restriction for' +
                    ' using only non-root node pools has been set (enforceNonRootNodes: true).\n' +
                    'Either contact your administrator or choose a non-root' +
                    ' node pool with operating' +
                    ' system Ubuntu 18 or Ubuntu 20.';
        }
        return {
            grpMessage,
            logMessage
        };
    }
    async skipSteps(group) {
        const update = {
            statusCode: metadata_1.SKIPPED,
            pendingLogsComplete: true,
            endedAt: new Date()
        };
        const pipelinesClient = new internal_clients_1.PipelinesClient(12);
        const stepUpdatePromises = group.groupStepIds.map((groupStepObject) => {
            return pipelinesClient.putStepById(groupStepObject.stepId, update);
        });
        await Promise.all(stepUpdatePromises);
    }
    async skipSteplets(group) {
        const update = {
            statusCode: metadata_1.SKIPPED,
            endedAt: new Date()
        };
        const pipelinesClient = new internal_clients_1.PipelinesClient(12);
        const stepletUpdatePromises = group.steplets.map((steplet) => {
            return pipelinesClient.putStepletById(steplet.id, update);
        });
        await Promise.all(stepletUpdatePromises);
        return;
    }
    async postStepSkippedConsoles(group) {
        const steplets = group.steplets;
        const stepletIds = (0, utility_1.map)(steplets, 'id');
        const consoleAdapter = new BatchStepletConsoleAdapter_js_1.default((0, utility_1.uuidv4)(), this.config.apiToken, stepletIds, group.pipelineId, this.config.jobConsoleBatchSize);
        const { grpMessage, logMessage } = this.getSkippedMessage(group);
        consoleAdapter.openGrp(grpMessage);
        consoleAdapter.openCmd('Info');
        consoleAdapter.publishMsg(logMessage);
        consoleAdapter.closeCmd(false);
        consoleAdapter.closeGrp(false);
        const flushConsolesPromisfied = (0, node_util_1.promisify)(consoleAdapter.flushConsoles).bind(consoleAdapter);
        await flushConsolesPromisfied();
    }
    async createToken(tokenRequest) {
        const accessClient = internal_clients_1.AccessClientFactory.getClient();
        const token = await accessClient.createAccessTokenRequest(tokenRequest);
        return token.accessToken;
    }
}
exports.StepScheduler = StepScheduler;
//# sourceMappingURL=StepScheduler.js.map