"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SendApprovalRequests = void 0;
const node_util_1 = require("node:util");
const utility_1 = require("@jfrog-pipelines/utility");
const microservice_1 = require("@jfrog-pipelines/microservice");
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const configuration_1 = require("@jfrog-pipelines/configuration");
const logger_1 = require("@jfrog-pipelines/logger");
const waitingGroupManager_1 = require("../waitingGroupManager");
const POST_APPROVAL_BATCH = 5;
const WAITING_STEP_ELIGIBILITY_BATCH = 10;
const REQUEST_APPROVAL_BATCH = 500;
const MS_NAME = 'step-service';
const LOCK_TIMEOUT_ONE_MIN_IN_MS = 60000;
class SendApprovalRequests {
    constructor(jFrogPipelinesSessionId) {
        this.logger = logger_1.LoggerFactory.getLogger();
        this.waitingGroupManager = new waitingGroupManager_1.WaitingGroupManager();
        this.systemInstanceId = microservice_1.applicationContext.get(microservice_1.SYSTEM_SERVICE_INSTANCE_INITIALIZER)
            ?.systemServiceInstancesId;
        this.properties = configuration_1.ConfigFactory.config;
        this.apiClient = new internal_clients_1.PipelinesClient(12, this.properties.apiToken);
        this.jFrogPipelinesSessionId = jFrogPipelinesSessionId;
        this.isAnyApprovalErrored = false;
    }
    async getWaitingAndRejectedGroupsAfterApprovalProcessing(waitingGroups) {
        const filteredWaitingGroups = this.waitingGroupManager.filterWaitingGroupsByApprovalRequired(waitingGroups);
        await this.postApprovalRequest(filteredWaitingGroups.groupsRequiringApproval);
        return {
            waitingGroups: filteredWaitingGroups.waitingGroupsResult,
            rejectedGroupsForApproval: filteredWaitingGroups.rejectedGroupsWaitingForApproval
        };
    }
    async postApprovalRequest(groupsRequiringApproval) {
        const groupsRequiringApprovalByProjectId = (0, utility_1.groupBy)(groupsRequiringApproval, 'projectId');
        await (0, utility_1.eachLimit)((0, utility_1.keys)(groupsRequiringApprovalByProjectId), POST_APPROVAL_BATCH, async (projectId) => {
            const projectIdNumber = Number.parseInt(projectId, 10);
            await this.processBatchRequest(projectIdNumber, groupsRequiringApprovalByProjectId);
        });
    }
    async processBatchRequest(projectId, groupsRequiringApprovalByProjectId) {
        const lockName = `${MS_NAME}-stepApprovalRequest-${projectId}`;
        const lock = await microservice_1.Lock.create(this.systemInstanceId, lockName, LOCK_TIMEOUT_ONE_MIN_IN_MS);
        try {
            if (lock === null) {
                this.logger.info(`Failed to request approvals for project: ${projectId} because lock Already Exists`);
                return;
            }
            await lock.obtainLock();
            const waitingGroups = groupsRequiringApprovalByProjectId[projectId] ?? [];
            let totalApprovalRequests = 0;
            if (waitingGroups.length > 0) {
                const waitingSteps = this.getWaitingStepsUsingRunAdjacency(waitingGroups);
                await this.checkIfStepsShouldBeSkipped(waitingSteps);
                totalApprovalRequests = await this.approveRequestsAndGetApprovalCount(waitingSteps);
            }
            this.logger.debug(`Requested approvals for steps in project: ${projectId}`);
            this.logEvent(totalApprovalRequests, projectId);
        }
        catch (error) {
            this.logger.warn(`Failed to request approvals for project: ${projectId} with error: ${(0, node_util_1.inspect)(error)}`);
        }
        finally {
            await lock?.releaseLock();
        }
    }
    getWaitingStepsUsingRunAdjacency(waitingGroups) {
        const waitingStepIds = [];
        for (const waitingGroup of waitingGroups) {
            const stepsIds = waitingGroup.getApprovalGatesStepIds();
            const waitingStepInfo = stepsIds.map((id) => {
                return { id: id, skipStep: false };
            });
            waitingStepIds.push(...waitingStepInfo);
        }
        return waitingStepIds;
    }
    async checkIfStepsShouldBeSkipped(waitingSteps) {
        let isAnyStepErrored = false;
        await (0, utility_1.eachLimit)(waitingSteps, WAITING_STEP_ELIGIBILITY_BATCH, async (waitingStep) => {
            try {
                const stepExecutionResult = await this.apiClient.getStepExecutionEligibility(waitingStep.id);
                if (!stepExecutionResult.canExecuteStep) {
                    this.logger.debug(`Step ${waitingStep.id} will be skipped`);
                    waitingStep.skipStep = true;
                }
            }
            catch (error) {
                this.logger.warn(`Failed to check execution eligibility of step ${waitingStep.id} with error: ${(0, node_util_1.inspect)(error)}`);
                isAnyStepErrored = true;
            }
        });
        if (isAnyStepErrored) {
            throw new Error('check execution eligibility failed');
        }
    }
    async approveRequestsAndGetApprovalCount(waitingSteps) {
        let approvalRequests = 0;
        await (0, utility_1.eachLimit)(waitingSteps, REQUEST_APPROVAL_BATCH, async (step) => {
            if (!step.skipStep) {
                try {
                    await this.apiClient.requestStepApproval(step.id);
                    approvalRequests += 1;
                }
                catch (error) {
                    this.logger.warn(`Failed to requestStepApproval for id:${step.id} with error: ${(0, node_util_1.inspect)(error)}`);
                    this.isAnyApprovalErrored = true;
                }
            }
        });
        return approvalRequests;
    }
    logEvent(totalApprovalRequests, projectId) {
        if (totalApprovalRequests > 0) {
            const eventData = {
                timestamp: new Date().toISOString(),
                domain: MS_NAME,
                eventType: 'approvalrequests',
                status: this.isAnyApprovalErrored ? 'failed' : 'success',
                noOfRequests: totalApprovalRequests,
                projectId: projectId,
                pipelinesSessionId: this.jFrogPipelinesSessionId
            };
            logger_1.LoggerFactory.logEvent(eventData);
        }
    }
}
exports.SendApprovalRequests = SendApprovalRequests;
//# sourceMappingURL=SendApprovalRequest.js.map