"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const microservice_1 = require("@jfrog-pipelines/microservice");
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const logger_1 = require("@jfrog-pipelines/logger");
const SendApprovalRequest_1 = require("../SendApprovalRequest");
const modules_1 = require("../../modules");
jest.mock('@jfrog-pipelines/configuration', () => {
    return {
        ConfigFactory: { config: { apiToken: 'apiToken' } }
    };
});
jest.mock('@jfrog-pipelines/logger', () => {
    const actualTracker = jest.requireActual('@jfrog-pipelines/logger');
    return {
        ...actualTracker,
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            logEvent: jest.fn()
        }
    };
});
const logger = logger_1.LoggerFactory.getLogger();
describe('SendApprovalRequest', () => {
    microservice_1.applicationContext.set(microservice_1.SYSTEM_SERVICE_INSTANCE_INITIALIZER, { systemServiceInstancesId: 1 });
    const sendRequestApprovalsStatus = new SendApprovalRequest_1.SendApprovalRequests('session');
    it('should get approve one step and one cannot execute', async () => {
        const waitingGroupMock = [
            {
                projectId: 1,
                requiresApproval: true,
                rejectionContext: {},
                getApprovalGatesStepIds: () => {
                    return [1, 2];
                }
            }
        ];
        jest.spyOn(modules_1.WaitingGroup.prototype, 'getApprovalGatesStepIds').mockReturnValueOnce([1, 2]);
        await sendRequestApprovalsStatus.getWaitingAndRejectedGroupsAfterApprovalProcessing(waitingGroupMock);
        expect(logger.info).toHaveBeenCalledTimes(1);
        expect(logger.info).toHaveBeenCalledWith(`Failed to request approvals for project: 1 because lock Already Exists`);
    });
    it('should give info logs when error occurred when trying to call create lock method', async () => {
        const waitingGroupMock = [
            {
                projectId: 1,
                requiresApproval: true,
                rejectionContext: {},
                getApprovalGatesStepIds: () => {
                    return [1, 2];
                }
            }
        ];
        const createLockSpy = jest.spyOn(microservice_1.Lock, 'create').mockResolvedValueOnce(null);
        await sendRequestApprovalsStatus.getWaitingAndRejectedGroupsAfterApprovalProcessing(waitingGroupMock);
        expect(createLockSpy).toHaveBeenCalledTimes(1);
        expect(logger.info).toHaveBeenCalledTimes(1);
        expect(logger.info).toHaveBeenCalledWith('Failed to request approvals for project: 1 because lock Already Exists');
    });
    it('should give info logs when obtain lock returned null', async () => {
        const waitingGroupMock = [
            {
                projectId: 1,
                requiresApproval: true,
                rejectionContext: {},
                getApprovalGatesStepIds: () => {
                    return [1, 2];
                }
            }
        ];
        const createLockSpy = jest.spyOn(microservice_1.Lock, 'create').mockResolvedValueOnce(null);
        const obtainLockSpy = jest.spyOn(microservice_1.Lock.prototype, 'obtainLock').mockResolvedValueOnce();
        const releaseLockSpy = jest.spyOn(microservice_1.Lock.prototype, 'releaseLock').mockResolvedValueOnce();
        await sendRequestApprovalsStatus.getWaitingAndRejectedGroupsAfterApprovalProcessing(waitingGroupMock);
        expect(logger.info).toHaveBeenCalledTimes(1);
        expect(createLockSpy).toHaveBeenCalledTimes(1);
        expect(obtainLockSpy).toHaveBeenCalledTimes(0);
        expect(releaseLockSpy).toHaveBeenCalledTimes(0);
        expect(logger_1.LoggerFactory.logEvent).toHaveBeenCalledTimes(0);
    });
    it('should give info logs when unable to check step execution eligibility', async () => {
        const waitingGroupMock = [
            {
                projectId: 1,
                requiresApproval: true,
                rejectionContext: {},
                getApprovalGatesStepIds: () => {
                    return [1, 2];
                }
            }
        ];
        jest.spyOn(internal_clients_1.PipelinesClient.prototype, 'getStepExecutionEligibility').mockRejectedValueOnce('error');
        await sendRequestApprovalsStatus.getWaitingAndRejectedGroupsAfterApprovalProcessing(waitingGroupMock);
        expect(logger.info).toHaveBeenCalledTimes(1);
        expect(logger_1.LoggerFactory.logEvent).toHaveBeenCalledTimes(0);
    });
    it('should log warning when unable to approve step and then log event', async () => {
        const waitingGroupMock = [
            {
                projectId: 1,
                requiresApproval: true,
                rejectionContext: {},
                getApprovalGatesStepIds: () => {
                    return [1, 2];
                }
            }
        ];
        jest.spyOn(internal_clients_1.PipelinesClient.prototype, 'getStepExecutionEligibility').mockResolvedValue({ canExecuteStep: true });
        jest.spyOn(internal_clients_1.PipelinesClient.prototype, 'requestStepApproval').mockRejectedValueOnce('errorApproval');
        jest.spyOn(internal_clients_1.PipelinesClient.prototype, 'requestStepApproval').mockResolvedValueOnce({});
        jest.spyOn(modules_1.WaitingGroup.prototype, 'getApprovalGatesStepIds').mockReturnValueOnce([1, 2]);
        await sendRequestApprovalsStatus.getWaitingAndRejectedGroupsAfterApprovalProcessing(waitingGroupMock);
        expect(logger.info).toHaveBeenCalledTimes(1);
    });
});
//# sourceMappingURL=SendApprovalRequest.test.js.map