"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CheckExecutionCapacity = void 0;
const node_util_1 = require("node:util");
const logger_1 = require("@jfrog-pipelines/logger");
const utility_1 = require("@jfrog-pipelines/utility");
const acl_1 = require("../acl");
const PROCESSING_STATUS_CODE = 4001;
const SUCCESS_STATUS_CODE = 4002;
const FAILURE_STATUS_CODE = 4003;
class CheckExecutionCapacity {
    constructor() {
        this.logger = logger_1.LoggerFactory.getLogger('CheckExecutionCapacity');
        this.hasRemainingCapacity = false;
        this.nodePoolAcl = new acl_1.NodePoolRepository();
        this.nodePools = [];
        this.nodes = [];
        this.nodePoolsGroupedByIsOnDemand = {};
        this.dynamicNodePools = [];
        this.staticNodePools = [];
        this.nodesByNodePoolId = {};
        this.availableNodePools = [];
    }
    async checkCapacity() {
        try {
            await this.checkLicenseCapacity();
            await this.getNodePools();
            await this.getNodes();
            this.filterAvailableNodePools();
            const result = {
                availableNodePools: this.availableNodePools,
                hasRemainingCapacity: this.hasRemainingCapacity,
                allNodePools: this.nodePools
            };
            return result;
        }
        catch (error) {
            this.logger.warn('failed to check execution capacity:', (0, node_util_1.inspect)(error));
            return {
                availableNodePools: [],
                allNodePools: [],
                hasRemainingCapacity: false
            };
        }
    }
    async checkLicenseCapacity() {
        try {
            const results = await this.nodePoolAcl.getExecutionNodes();
            if (results.available > 0) {
                this.hasRemainingCapacity = true;
            }
        }
        catch (error) {
            this.logger.warn('Cannot get executable nodes. Exiting! Error:', (0, node_util_1.inspect)(error));
            throw error;
        }
    }
    async getNodePools() {
        try {
            this.nodePools = await this.nodePoolAcl.getNodePools();
            this.nodePoolsGroupedByIsOnDemand = (0, utility_1.groupBy)(this.nodePools, 'isOnDemand');
            this.dynamicNodePools = this.nodePoolsGroupedByIsOnDemand.true;
            this.staticNodePools = this.nodePoolsGroupedByIsOnDemand.false;
        }
        catch (error) {
            this.logger.warn('Cannot get nodePools, Exiting error:', (0, node_util_1.inspect)(error));
            throw error;
        }
    }
    async getNodes() {
        if ((0, utility_1.isEmpty)(this.nodePools)) {
            this.logger.warn('Skipping because of no nodePools found');
            return;
        }
        const queryToGetNodes = this.generateQueryToGetNodes();
        try {
            this.nodes = await this.nodePoolAcl.getNodes(queryToGetNodes);
            this.nodesByNodePoolId = (0, utility_1.groupBy)(this.nodes, 'nodePoolId');
        }
        catch (error) {
            this.logger.warn(`Failed to getNodes for ${(0, node_util_1.inspect)(queryToGetNodes)}: %s with ${(0, node_util_1.inspect)(error)} `);
            throw error;
        }
    }
    generateQueryToGetNodes() {
        return `nodePoolIds=${(0, utility_1.map)(this.nodePools, 'id').join(',')}`;
    }
    filterAvailableNodePools() {
        if ((0, utility_1.isEmpty)(this.nodePools)) {
            this.logger.warn('No available nodePools');
            return;
        }
        this.filterAvailableStaticNodePools();
        this.filterAvailableDynamicNodePools();
    }
    filterAvailableStaticNodePools() {
        if ((0, utility_1.isEmpty)(this.staticNodePools)) {
            return;
        }
        const nodePoolIds = Object.keys(this.nodesByNodePoolId);
        for (const staticNodePool of this.staticNodePools) {
            const staticNodes = this.getStaticNodesByNodePoolId(nodePoolIds, staticNodePool);
            const areNodesAvailable = (0, utility_1.isEmpty)(staticNodes);
            const areSuccessNodes = this.getSuccessNodes(staticNodes);
            const areFailedNodes = this.getFailureNodes(staticNodes);
            this.pushAvailableStaticNodes(areNodesAvailable, areSuccessNodes, areFailedNodes, staticNodePool);
        }
    }
    getStaticNodesByNodePoolId(nodePoolIds, staticNodePool) {
        if (nodePoolIds.includes(staticNodePool.id.toString())) {
            return this.nodesByNodePoolId[staticNodePool.id];
        }
        return [];
    }
    getSuccessNodes(staticNodes) {
        const areSuccessNodes = [];
        for (const node of staticNodes) {
            if (node.statusCode === SUCCESS_STATUS_CODE) {
                areSuccessNodes.push(node);
                break;
            }
        }
        return areSuccessNodes;
    }
    getFailureNodes(staticNodes) {
        const areFailedNodes = [];
        for (const node of staticNodes) {
            if (node.statusCode === FAILURE_STATUS_CODE) {
                areFailedNodes.push(node);
            }
        }
        return areFailedNodes;
    }
    pushAvailableStaticNodes(areNodesAvailable, areSuccessNodes, areFailedNodes, staticNodePool) {
        if (areSuccessNodes || !areNodesAvailable || areFailedNodes) {
            this.availableNodePools.push(staticNodePool);
        }
    }
    filterAvailableDynamicNodePools() {
        if ((0, utility_1.isEmpty)(this.dynamicNodePools)) {
            return;
        }
        const nodePoolIds = Object.keys(this.nodesByNodePoolId);
        for (const dynamicNodePool of this.dynamicNodePools) {
            const maxNodesAllowed = dynamicNodePool.numberOfNodes;
            const dynamicNodes = this.getDynamicNodesByNodePoolId(nodePoolIds, dynamicNodePool);
            const processingNodes = dynamicNodes.filter((node) => {
                return node.statusCode === PROCESSING_STATUS_CODE;
            });
            this.pushAvailableDynamicNodes(maxNodesAllowed, processingNodes, dynamicNodePool);
        }
    }
    getDynamicNodesByNodePoolId(nodePoolIds, dynamicNodePool) {
        if (nodePoolIds.includes(dynamicNodePool.id.toString())) {
            return this.nodesByNodePoolId[dynamicNodePool.id];
        }
        return [];
    }
    pushAvailableDynamicNodes(maxNodesAllowed, processingNodes, dynamicNodePool) {
        if (maxNodesAllowed > 0 && maxNodesAllowed > processingNodes.length) {
            this.availableNodePools.push(dynamicNodePool);
        }
    }
}
exports.CheckExecutionCapacity = CheckExecutionCapacity;
//# sourceMappingURL=checkExecutionCapacityNew.js.map