'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
var self = prepData;
module.exports = self;
var _ = require('underscore');
var async = require('async');
var { getElapsedTimeSinceStartTime } = require('./utils');
var getDownstreamStepIds = require('./getDownStreamStepIds.js');
const { waitingConditions } = require('./constants');
var { CALL_OBJECT_LIMIT, CALL_OBJECT_LIMIT_FOR_BATCH } = require('../constants.js');
const isExceededBuildMinutesQuota = require('../IsExceededBuildMinutesQuota');
const BatchStepletConsoleAdapter = require('../BatchStepletConsoleAdapter');
const { DataAdapterFactory } = require('../modules/dataAdapter/DataAdapterFactory');
const { AffinityGroupManager } = require('../modules/affinityGroup/AffinityGroupManager');
const logger_1 = require("@jfrog-pipelines/logger");
const acl_1 = require("../acl");
const utility_1 = require("@jfrog-pipelines/utility");
function prepData(externalBag, callback) {
    var who = externalBag.who + '|' + prepData.name;
    logger.debug(who, 'Inside');
    var bag = {
        who: who,
        suAdapter: externalBag.suAdapter,
        statusCodes: global.systemCodesByGroup.status,
        hasRemainingCapacity: externalBag.hasRemainingCapacity,
        availableNodePools: externalBag.availableNodePools,
        allNodePools: externalBag.allNodePools,
        defaultPriority: externalBag.defaultPriority,
        jFrogPipelinesSessionId: externalBag.jFrogPipelinesSessionId,
        skipByBuildLimitCrossed: false,
        externalStepTypes: ['Jenkins'],
        rejectedGroups: externalBag.rejectedGroups,
        waitingGroups: [],
        processingGroups: [],
        result: {},
    };
    let startTime = Date.now();
    let tasks = [
        _checkInputParams.bind(null, bag),
        _mapAffinityGroupsFromRunAdjacency.bind(null, bag),
        _isExceededBuildMinutesQuota.bind(null, bag),
        _enforceBuildMinutesLimit.bind(null, bag),
        _getNodePools.bind(null, bag),
        _rejectWaitingGroupsByNodePools.bind(null, bag),
        _getGroupsProjects.bind(null, bag),
        _getProjectsAllowedNodePools.bind(null, bag),
        _getOrphanedNodes.bind(null, bag),
        _setResult.bind(null, bag)
    ];
    async.series(tasks, function (err) {
        let prepDataStatus = 'success';
        if (err)
            prepDataStatus = 'failed';
        var stopTime = Date.now();
        var timeDiff = new Date(stopTime - startTime);
        var finalTime = util.format("durationMins=%s - %s minutes, %s seconds, and " +
            "%s milliseconds", (timeDiff.getTime() / 1000 / 60), timeDiff.getMinutes(), timeDiff.getSeconds(), timeDiff.getMilliseconds());
        logger.debug(bag.who, 'Finished prepData -', finalTime);
        const eventData = {
            timestamp: new Date(stopTime).toISOString(),
            domain: global.msName,
            eventType: 'prepdata',
            status: prepDataStatus,
            durationMillis: timeDiff.valueOf(),
            pipelinesSessionId: bag.jFrogPipelinesSessionId
        };
        logger_1.LoggerFactory.logEvent(eventData);
        return callback(err, bag.result);
    });
}
function _checkInputParams(bag, next) {
    var who = bag.who + '|' + _checkInputParams.name;
    logger.debug(who, 'Inside');
    if (!bag.suAdapter)
        return next(new Error(':suAdapter is missing'));
    if (!bag.hasRemainingCapacity && _.isEmpty(bag.availableNodePools))
        bag.skip = true;
    return next();
}
function _isExceededBuildMinutesQuota(bag, next) {
    const who = bag.who + '|' + _isExceededBuildMinutesQuota.name;
    logger.trace(who, 'Inside');
    const internalBag = {
        jFrogPipelinesSessionId: bag.jFrogPipelinesSessionId
    };
    isExceededBuildMinutesQuota(internalBag, function (err, result) {
        if (err) {
            logger.debug(who, `Failed to verify whether build minutes quota was exceeded with error: ${err.message}`);
            return next();
        }
        bag.skipByBuildLimitCrossed = result;
        return next();
    });
}
function _enforceBuildMinutesLimit(bag, next) {
    if (!bag.skipByBuildLimitCrossed)
        return next();
    const who = bag.who + '|' + _enforceBuildMinutesLimit.name;
    logger.trace(who, 'Inside');
    async.series([
        __postStepletCancellationMessages.bind(null, bag),
        __cancelIncompleteRuns.bind(null, bag)
    ], function (err) {
        if (err) {
            logger.warn(who, `Enforcing build minutes quota failed with error: ${err.message}`);
        }
        bag.waitingGroups = [];
        return next();
    });
}
function __postStepletCancellationMessages(bag, next) {
    const who = bag.who + '|' + __postStepletCancellationMessages.name;
    logger.trace(who, 'Inside');
    const incompleteStepletsByPipelineId = Object.entries(_.groupBy(bag.incompleteSteplets, 'pipelineId'));
    async.eachLimit(incompleteStepletsByPipelineId, 5, function (incompleteStepletsForPipelineId, done) {
        ___postStepCancelledConsoles({
            who: bag.who,
            pipelineId: incompleteStepletsForPipelineId[0],
            stepletIds: _.pluck(incompleteStepletsForPipelineId[1], 'id'),
            message: 'Cancelling step as build minutes quota was exceeded',
            jFrogPipelinesSessionId: bag.jFrogPipelinesSessionId
        });
        return done();
    }, function (err) {
        if (err) {
            logger.warn(who, `Failed to post step cancelled console message: ${err.message}`);
        }
        return next();
    });
}
function __cancelIncompleteRuns(bag, next) {
    const who = bag.who + '|' + __cancelIncompleteRuns.name;
    logger.trace(who, 'Inside');
    const incompleteGroups = bag.processingGroups.concat(bag.waitingGroups);
    const runIds = [...new Set(incompleteGroups.map(incompleteGroup => incompleteGroup.runId))];
    async.eachLimit(runIds, 5, async function (runId, done) {
        const cancelRunByIdAsync = util.promisify(bag.suAdapter.cancelRunById.bind(bag.suAdapter));
        try {
            await cancelRunByIdAsync(runId);
        }
        catch (err) {
            return done(err);
        }
        return done();
    }, function (err) {
        if (err) {
            logger.warn(who, 'Failed to cancel run', err);
            return next(err);
        }
        return next();
    });
}
function ___postStepCancelledConsoles(bag) {
    const who = bag.who + '|' + ___postStepCancelledConsoles.name;
    logger.debug(who, 'Inside');
    const consoleAdapter = new BatchStepletConsoleAdapter(bag.jFrogPipelinesSessionId, global.config.apiToken, bag.stepletIds, bag.pipelineId, global.systemConfig.jobConsoleBatchSize, global.systemConfig.jobConsoleBufferTimeInterval);
    consoleAdapter.openGrp('Step cancelled');
    consoleAdapter.openCmd('Info');
    consoleAdapter.publishMsg(bag.message);
    consoleAdapter.closeCmd(false);
    consoleAdapter.closeGrp(false);
}
function _mapAffinityGroupsFromRunAdjacency(bag, next) {
    var who = bag.who + '|' + _mapAffinityGroupsFromRunAdjacency.name;
    logger.debug(who, 'Inside');
    const affinityGroupManager = new AffinityGroupManager(DataAdapterFactory.getApiDataAdapter());
    affinityGroupManager.getAffinityGroups().then((affinityGroups) => {
        bag.waitingGroups = affinityGroups;
        return next();
    }).catch(err => {
        return next(err);
    });
}
function _rejectWaitingGroupsByNodePools(bag, next) {
    if (_.isEmpty(bag.waitingGroups))
        return next();
    var who = bag.who + '|' + _rejectWaitingGroupsByNodePools.name;
    logger.debug(who, 'Inside');
    const rejectWaitingGroupsByNodePools = new acl_1.RejectWaitingGroupsByNodePools();
    const response = rejectWaitingGroupsByNodePools.rejectWaitingGroupsByNodePools(bag.availableNodePools, bag.allNodePools, bag.waitingGroups);
    bag.rejectedGroups.waitingForFreeNodePoolCapacity = response.waitingForFreeNodePoolCapacity;
    bag.waitingGroups = response.waitingGroupsAfterFiltering;
    return next();
}
function _getGroupsProjects(bag, next) {
    if (_.isEmpty(bag.waitingGroups) &&
        _.isEmpty(bag.processingGroups))
        return next();
    var who = bag.who + '|' + _getGroupsProjects.name;
    logger.debug(who, 'Inside');
    const integrationRepository = new acl_1.IntegrationRepository();
    const projectIds = (0, utility_1.uniq)((0, utility_1.map)([...bag.processingGroups, ...bag.waitingGroups], 'projectId'));
    integrationRepository.getGroupProjects(projectIds)
        .then((projects) => {
        bag.groupsProjects = projects;
        return next();
    })
        .catch((err) => {
        logger.warn(err);
        return next(err);
    });
}
function _getProjectsAllowedNodePools(bag, next) {
    if (_.isEmpty(bag.groupsProjects))
        return next();
    var who = bag.who + '|' + _getProjectsAllowedNodePools.name;
    logger.debug(who, 'Inside');
    bag.allowedNodePoolsByProject = {};
    const nodePoolRepository = new acl_1.NodePoolRepository();
    nodePoolRepository.getProjectsAllowedNodePools(bag.groupsProjects)
        .then((allowedNodePoolsByProject) => {
        bag.allowedNodePoolsByProject = allowedNodePoolsByProject;
        return next();
    })
        .catch((err) => {
        return next(err);
    });
}
function _getNodePools(bag, next) {
    var who = bag.who + '|' + _getNodePools.name;
    logger.debug(who, 'Inside');
    const nodePoolRepository = new acl_1.NodePoolRepository();
    return nodePoolRepository.getNodePools()
        .then((nodePools) => {
        bag.nodePools = nodePools;
        return next();
    })
        .catch((err) => {
        logger.warn(who, util.format('Failed to getNodePools with error: %s', err));
        return next(true);
    });
}
function _getOrphanedNodes(bag, next) {
    var who = bag.who + '|' + _getOrphanedNodes.name;
    logger.debug(who, 'Inside');
    const nodePoolRepository = new acl_1.NodePoolRepository();
    return nodePoolRepository.getOrphanedNodes()
        .then((response) => {
        bag.orphanedOnDemandNodesByProjectId = response.orphanedOnDemandNodesByProjectId;
        bag.nodePools.concat(response.orphanedNodePools);
        return next();
    })
        .catch((err) => {
        logger.warn(err);
        return next(true);
    });
}
function _setResult(bag, next) {
    var who = bag.who + '|' + _setResult.name;
    logger.debug(who, 'Inside');
    bag.result = {
        incompleteStepsById: bag.incompleteStepsById,
        incompleteStepletsById: bag.incompleteStepletsById,
        incompleteStepletsByStepId: bag.incompleteStepletsByStepId,
        incompleteStepsByRunId: bag.incompleteStepsByRunId,
        waitingGroups: bag.waitingGroups,
        processingGroups: bag.processingGroups,
        pipelines: bag.pipelines,
        pipelinesById: bag.pipelinesById,
        runStepConnections: bag.runStepConnections,
        runStepConnectionsByStepId: bag.runStepConnectionsByStepId,
        runStepConnectionsByRunId: bag.runStepConnectionsByRunId,
        runResourceVersionsById: bag.runResourceVersionsById,
        runResourceVersionsByRunId: bag.runResourceVersionsByRunId,
        groupsProjects: bag.groupsProjects,
        allowedNodePoolsByProject: bag.allowedNodePoolsByProject,
        nodePools: bag.nodePools,
        orphanedOnDemandNodesByProjectId: bag.orphanedOnDemandNodesByProjectId,
        runsById: bag.runsById,
        rejectedGroups: bag.rejectedGroups
    };
    return next();
}
//# sourceMappingURL=prepData.js.map