"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.QueueExternalSteps = void 0;
const node_util_1 = require("node:util");
const utility_1 = require("@jfrog-pipelines/utility");
const microservice_1 = require("@jfrog-pipelines/microservice");
const logger_1 = require("@jfrog-pipelines/logger");
const waitingStepsByRunAdjacency_1 = require("../waitingStepsByRunAdjacency");
const waitingGroupManager_1 = require("../waitingGroupManager");
const tryAndQueueExternalStepsNew_1 = require("./tryAndQueueExternalStepsNew");
const MS_NAME = 'stepService';
const SIXTY = 60;
const THOUSAND = 1000;
const ONE = 1;
const LOCK_TIMEOUT_IN_MS = ONE * SIXTY * THOUSAND;
class QueueExternalSteps {
    constructor(groups, jfrogPipelinesSessionId, apiToken) {
        this.jfrogPipelinesSessionId = jfrogPipelinesSessionId;
        this.apiToken = apiToken;
        this.logger = logger_1.LoggerFactory.getLogger('QueueExternalSteps');
        this.waitingGroups = groups;
        this.sessionId = jfrogPipelinesSessionId;
        this.apiToken = apiToken;
        this.systemInstanceId = microservice_1.applicationContext.get(microservice_1.SYSTEM_SERVICE_INSTANCE_INITIALIZER)
            ?.systemServiceInstancesId;
    }
    async queueExternalSteps() {
        if ((0, utility_1.isEmpty)(this.waitingGroups)) {
            return;
        }
        const waitingExternalGroupsByProjectId = this.getWaitingExternalGroupsByProjectId();
        const projectIds = Object.keys(waitingExternalGroupsByProjectId);
        const queueExternalStepsPromises = projectIds.map((id) => {
            const groups = waitingExternalGroupsByProjectId[id];
            const projectId = Number.parseInt(id, 10);
            return this.queueExternalStepsWorker(groups, projectId);
        });
        await Promise.all(queueExternalStepsPromises);
    }
    async queueExternalStepsWorker(waitingGroups, projectId) {
        const name = `${MS_NAME}-externalSteps-${projectId}`;
        const lock = await microservice_1.Lock.create(this.systemInstanceId, name, LOCK_TIMEOUT_IN_MS);
        try {
            if (lock === null) {
                this.logger.info(`Cannot queue external Step with projectId: ${projectId} because lock Already Exists with name ${name}`);
                return;
            }
            await this.obtainExternalStepsLock(lock);
            await this.processExternalSteps(waitingGroups, lock);
        }
        catch (error) {
            this.logger.warn(`Cannot queue external Step with projectId: ${projectId}. Faced Error: ${(0, node_util_1.inspect)(error)}`);
        }
        finally {
            if (lock) {
                await lock.releaseLock();
            }
        }
    }
    async obtainExternalStepsLock(lock) {
        try {
            await lock.obtainLock();
        }
        catch (error) {
            throw new Error((0, node_util_1.inspect)(error));
        }
    }
    async getWaitingExternalSteps(waitingGroups) {
        if ((0, utility_1.isEmpty)(waitingGroups)) {
            this.logger.warn('No groups to process. Exiting!');
            return [];
        }
        try {
            const filteredWaitingGroups = await waitingStepsByRunAdjacency_1.WaitingStepsByRunAdjacency.getWaitingStepsByRunAdjacencies(waitingGroups);
            return filteredWaitingGroups;
        }
        catch (error) {
            this.logger.warn('Cannot get external filtered waiting groups by run adjacency. Exiting! error:', (0, node_util_1.inspect)(error));
            throw new Error((0, node_util_1.inspect)(error));
        }
    }
    async setExternalStepsReadyAt(waitingGroups) {
        if ((0, utility_1.isEmpty)(this.waitingGroups)) {
            this.logger.warn('No groups to process. Exiting!');
            return;
        }
        try {
            const waitingGroupManager = new waitingGroupManager_1.WaitingGroupManager();
            await waitingGroupManager.setReadyAt(waitingGroups);
        }
        catch (error) {
            this.logger.warn('Cannot update readyAt value for the waiting group. Exiting! Error:', (0, node_util_1.inspect)(error));
            throw new Error((0, node_util_1.inspect)(error));
        }
    }
    async tryAndQueueExternalSteps(waitingGroup) {
        if ((0, utility_1.isEmpty)(this.waitingGroups)) {
            this.logger.warn('No groups to process. Exiting!');
            return;
        }
        try {
            const queueExternalSteps = new tryAndQueueExternalStepsNew_1.TryAndQueueExternalSteps(waitingGroup, this.sessionId, this.apiToken);
            await queueExternalSteps.queueSteps();
        }
        catch (error) {
            this.logger.warn('Cannot queue External Step. Exiting! Error:', (0, node_util_1.inspect)(error));
        }
    }
    getWaitingExternalGroupsByProjectId() {
        const externalWaitingGroups = [];
        for (const group of this.waitingGroups) {
            if (group.isExternal) {
                externalWaitingGroups.push(group);
            }
        }
        const waitingExternalGroupsByProjectId = (0, utility_1.groupBy)(externalWaitingGroups, 'projectId');
        return waitingExternalGroupsByProjectId;
    }
    async processExternalSteps(waitingGroups, lock) {
        if (!lock) {
            this.logger.info(`Invalid lock found`);
            return;
        }
        try {
            const filteredWaitingGroups = await this.getWaitingExternalSteps(waitingGroups);
            await this.setExternalStepsReadyAt(filteredWaitingGroups);
            await this.tryAndQueueExternalSteps(filteredWaitingGroups);
        }
        catch (error) {
            this.logger.warn(`Cannot process external Steps because of error: ${(0, node_util_1.inspect)(error)}`);
            throw error;
        }
    }
}
exports.QueueExternalSteps = QueueExternalSteps;
//# sourceMappingURL=queueExternalStepsNew.js.map