"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodePoolRepository = void 0;
const node_util_1 = require("node:util");
const microservice_1 = require("@jfrog-pipelines/microservice");
const utility_1 = require("@jfrog-pipelines/utility");
const logger_1 = require("@jfrog-pipelines/logger");
const NodePoolDataAdapterFactory_1 = require("./NodePoolDataAdapterFactory");
class NodePoolRepository {
    constructor() {
        this.logger = logger_1.LoggerFactory.getLogger('NodePoolRepository');
        this.adapter = NodePoolDataAdapterFactory_1.NodePoolDataAdapterFactory.create(microservice_1.AclAdapterDi.api);
    }
    async getNodePools(query = '') {
        return this.adapter.getNodePoolsByQuery(query);
    }
    async getNodes(query = '') {
        return this.adapter.getNodesByQuery(query);
    }
    async getProjectsAllowedNodePools(groupProjects) {
        const allowedNodePoolsByProject = {};
        const groupProjectsBatchSize = 10;
        await (0, utility_1.eachLimit)(groupProjects, groupProjectsBatchSize, async (project) => {
            const query = `projectKeys=${project.sourceId}`;
            try {
                const nodePools = await this.adapter.getNodePoolsByQuery(query);
                allowedNodePoolsByProject[project.id] = nodePools;
            }
            catch (error) {
                this.logger.warn(`Failed to getNodePools for project ${project.sourceId} with error: ${error}`);
            }
        });
        return allowedNodePoolsByProject;
    }
    async getOrphanedNodes() {
        let nodes = [];
        try {
            nodes = await this.adapter.getNodesByQuery();
        }
        catch (error) {
            this.logger.warn(`Failed to getNodes with error: ${error}`);
            return {
                orphanedOnDemandNodesByProjectId: [],
                orphanedNodePools: []
            };
        }
        const orphanedOnDemandNodes = this.getOrphanedOnDemandNodes(nodes);
        if ((0, utility_1.isEmpty)(orphanedOnDemandNodes)) {
            return {
                orphanedOnDemandNodesByProjectId: [],
                orphanedNodePools: []
            };
        }
        const orphanedOnDemandNodesByProjectId = this.getOrphanedOnDemandNodesByProjectId(orphanedOnDemandNodes);
        const orphanedNodePools = this.getOrphanedNodePools(orphanedOnDemandNodesByProjectId);
        const response = {
            orphanedOnDemandNodesByProjectId: orphanedOnDemandNodesByProjectId,
            orphanedNodePools: orphanedNodePools
        };
        return response;
    }
    async getAndFilterAvailableNodes(nodePoolId, statusCodes) {
        const query = `nodePoolIds=${nodePoolId}`;
        try {
            const nodes = await this.getNodes(query);
            const filteredNodes = this.filterAvailableNodes(nodes, statusCodes);
            return filteredNodes;
        }
        catch (error) {
            this.logger.warn(`Cannot get nodes for the query: ${query} with error: ${error}`);
            throw new Error(error);
        }
    }
    async getBuildPlaneMessagesCount(status, nodePoolId) {
        const statusCodes = [status.ready.code, status.processing.code, status.queued.code];
        const query = `nodePoolIds=${nodePoolId}&statusCodes=${statusCodes.join(',')}`;
        try {
            const count = await this.adapter.getBuildPlaneMessagesCountByQuery(query);
            return count;
        }
        catch (error) {
            this.logger.warn(`Cannot get buildPlane messages count for the query:${query} with error:${(0, node_util_1.inspect)(error)}`);
            throw new Error((0, node_util_1.inspect)(error));
        }
    }
    findAvailableNodeCount(isOnDemandNodepool, processingGroups, useRunAdjacency, buildPlaneMessagesCount, numberOfNodesNodepool, statusCodes, nodes, isCacheEnabledNodePool) {
        let totalGroupsThatCanBeQueued = 0;
        let processingGroupsCount = 0;
        if (isOnDemandNodepool) {
            processingGroupsCount = this.getProcessingGroupsCount(processingGroups);
            if (useRunAdjacency) {
                processingGroupsCount = buildPlaneMessagesCount;
            }
            totalGroupsThatCanBeQueued = numberOfNodesNodepool - processingGroupsCount;
            const activeStatusCodes = [
                statusCodes.waiting.code,
                statusCodes.queued.code,
                statusCodes.processing.code,
                statusCodes.success.code
            ];
            const activeNodes = this.getActiveNodes(nodes, activeStatusCodes);
            const cachedNodes = this.getCachedNodes(nodes, statusCodes.cached.code, isCacheEnabledNodePool);
            const activeNodesCount = activeNodes.length;
            const cachedNodesCount = cachedNodes.length;
            return {
                totalGroupsThatCanBeQueued: totalGroupsThatCanBeQueued,
                cachedNodes: cachedNodes,
                activeNodesCount: activeNodesCount,
                cachedNodesCount: cachedNodesCount
            };
        }
        const processingNodes = this.getProcessingNodes(nodes, statusCodes.processing.code);
        const totalNodeCount = this.getTotalNodeCount(numberOfNodesNodepool, nodes);
        totalGroupsThatCanBeQueued = totalNodeCount - processingNodes.length;
        return {
            totalGroupsThatCanBeQueued: totalGroupsThatCanBeQueued,
            cachedNodes: [],
            cachedNodesCount: 0,
            activeNodesCount: processingNodes.length
        };
    }
    async getExecutionNodes() {
        return this.adapter.getExecutionNodes();
    }
    getProcessingGroupsCount(processingGroups) {
        return ((0, utility_1.countBy)(processingGroups, (group) => {
            return group.isExternal ? 'external' : 'notExternal';
        }).notExternal || 0);
    }
    getActiveNodes(nodes, activeStatusCodes) {
        const activeNodes = nodes.filter((node) => {
            return activeStatusCodes.includes(node.statusCode) && !node.stopRequested;
        });
        return activeNodes;
    }
    getCachedNodes(nodes, cachedCode, isCacheEnabledNodePool) {
        let cachedNodes = nodes.filter((node) => {
            return isCacheEnabledNodePool && node.statusCode === cachedCode && !node.stopRequested;
        });
        cachedNodes = (0, utility_1.sortBy)(cachedNodes, (cachedNode) => {
            return new Date(cachedNode.statusLastUpdatedAt);
        });
        return cachedNodes;
    }
    getProcessingNodes(nodes, processingCode) {
        const processingNodes = nodes.filter((node) => {
            return node.statusCode === processingCode && !node.stopRequested;
        });
        return processingNodes;
    }
    getTotalNodeCount(numberOfNodesNodePool, nodes) {
        return Math.min(nodes.length, numberOfNodesNodePool);
    }
    filterAvailableNodes(nodes, statusCodes) {
        const filteredNodes = nodes.filter((node) => {
            return this.isAllowedNode(node, statusCodes);
        });
        return filteredNodes;
    }
    isAllowedNode(node, statusCodes) {
        if (this.isWaitingOrQueuedNode(node, statusCodes)) {
            return true;
        }
        if (this.isProcessingNode(node, statusCodes)) {
            return true;
        }
        if (this.isSuccessOrCachedNode(node, statusCodes)) {
            return true;
        }
        return false;
    }
    isWaitingOrQueuedNode(node, statusCodes) {
        if (node.statusCode === statusCodes.waiting.code || node.statusCode === statusCodes.queued.code) {
            return true;
        }
        return false;
    }
    isProcessingNode(node, statusCodes) {
        if (node.statusCode === statusCodes.processing.code) {
            return true;
        }
        return false;
    }
    isSuccessOrCachedNode(node, statusCodes) {
        return this.isSuccessNode(node, statusCodes) || this.isCachedNode(node, statusCodes);
    }
    isSuccessNode(node, statusCodes) {
        if (node.statusCode === statusCodes.success.code) {
            return true;
        }
        return false;
    }
    isCachedNode(node, statusCodes) {
        if (node.statusCode === statusCodes.cached.code) {
            return true;
        }
        return false;
    }
    getOrphanedOnDemandNodes(nodes) {
        const orphanedOnDemandNodes = [];
        for (const node of nodes) {
            if (node.isOnDemand && !node.nodePoolId) {
                orphanedOnDemandNodes.push(node);
            }
        }
        return orphanedOnDemandNodes;
    }
    getOrphanedOnDemandNodesByProjectId(orphanedOnDemandNodes) {
        const orphanedOnDemandNodeByProjectId = {};
        for (const node of orphanedOnDemandNodes) {
            const projectId = node.projectId;
            if (!Object.keys(orphanedOnDemandNodeByProjectId).includes(projectId.toString())) {
                orphanedOnDemandNodeByProjectId[node.projectId] = [node];
            }
            else {
                orphanedOnDemandNodeByProjectId[node.projectId].push(node);
            }
        }
        return orphanedOnDemandNodeByProjectId;
    }
    getOrphanedNodePools(orphanedOnDemandNodesByProjectId) {
        const orphanedNodePools = [];
        const projectIds = Object.keys(orphanedOnDemandNodesByProjectId);
        for (const projectId of projectIds) {
            orphanedNodePools.push({
                name: 'orphaned#on-demand#nodepool',
                projectId: Number.parseInt(projectId, 10),
                isOrphaned: true,
                isOnDemand: true
            });
        }
        return orphanedNodePools;
    }
}
exports.NodePoolRepository = NodePoolRepository;
//# sourceMappingURL=NodePoolRepository.js.map