"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ApiNodePoolDataAdapter_1 = require("../ApiNodePoolDataAdapter");
const NodePoolRepository_1 = require("../NodePoolRepository");
jest.mock('@jfrog-pipelines/configuration', () => ({
    ConfigFactory: { config: { apiUrl: 'http://example.com', controlPlaneVersion: '1.23.6', defaultBatchSize: 100 } }
}));
const mockGetNodePoolsByQuery = (query = '') => {
    let result = [];
    if (query === 'projectKeys=100') {
        result = [{ id: 2 }, { id: 3 }, { id: 5 }];
    }
    else if (query === 'projectKeys=50') {
        result = [{ id: 20 }, { id: 30 }, { id: 50 }];
    }
    return Promise.resolve(result);
};
const mockGetNodePoolsByQueryThrowError = () => {
    throw new Error('error');
};
const mockErrorOutGetNodesByQuery = () => {
    throw new Error('Cannot extract Nodes according to the given query');
};
describe('NodePoolRepository test cases', () => {
    const nodePoolRepository = new NodePoolRepository_1.NodePoolRepository();
    const adapterClass = ApiNodePoolDataAdapter_1.ApiNodePoolDataAdapter;
    describe('getNodePools test cases', () => {
        it('should return all nodePools when called without any arguments', async () => {
            const spy = jest
                .spyOn(adapterClass.prototype, 'getNodePoolsByQuery')
                .mockResolvedValueOnce([{ id: 1 }, { id: 2 }]);
            const result = await nodePoolRepository.getNodePools();
            expect(result).toBeDefined();
            expect(spy).toHaveBeenCalledTimes(1);
        });
        it('should return nodePool for the nodePool when called with a query for the nodePoolId', async () => {
            const spy = jest
                .spyOn(adapterClass.prototype, 'getNodePoolsByQuery')
                .mockResolvedValueOnce([{ id: 1 }]);
            const query = 'nodePoolIds=1';
            const result = await nodePoolRepository.getNodePools(query);
            const mockExpectedResult = [{ id: 1 }];
            expect(result).toBeDefined();
            expect(spy).toHaveBeenCalledTimes(1);
            expect(result).toStrictEqual(mockExpectedResult);
        });
    });
    describe('getProjectsAllowedNodePools', () => {
        it('if no groupProjects are passed, an empty response should be returned', async () => {
            const adapterClass = ApiNodePoolDataAdapter_1.ApiNodePoolDataAdapter;
            const mockGetProjectAllowedNodePoolsResult = {};
            const input = [];
            const spy = jest.spyOn(adapterClass.prototype, 'getNodePoolsByQuery').mockImplementation((query) => {
                return mockGetNodePoolsByQuery(query);
            });
            const result = await nodePoolRepository.getProjectsAllowedNodePools(input);
            expect(spy).toHaveBeenCalledTimes(0);
            expect(result).toStrictEqual(mockGetProjectAllowedNodePoolsResult);
        });
        it('if a list of group project is provided, a response of allowed node pools mapped with projectId should be returned', async () => {
            const input = [
                { id: 1, sourceId: 100 },
                { id: 2, sourceId: 50 }
            ];
            const mockGetProjectAllowedNodePoolsResult = {
                '1': [{ id: 2 }, { id: 3 }, { id: 5 }],
                '2': [{ id: 20 }, { id: 30 }, { id: 50 }]
            };
            const spy = jest.spyOn(adapterClass.prototype, 'getNodePoolsByQuery').mockImplementation((query) => {
                return mockGetNodePoolsByQuery(query);
            });
            const result = await nodePoolRepository.getProjectsAllowedNodePools(input);
            expect(result).toBeDefined();
            expect(spy).toHaveBeenCalledTimes(input.length);
            expect(result).toStrictEqual(mockGetProjectAllowedNodePoolsResult);
        });
        it('if one group project is provided, a response of allowed node pools mapped with projectId should be returned', async () => {
            const input = [{ id: 1, sourceId: 100 }];
            const mockGetProjectAllowedNodePoolsResult = {
                '1': [{ id: 2 }, { id: 3 }, { id: 5 }]
            };
            const spy = jest.spyOn(adapterClass.prototype, 'getNodePoolsByQuery').mockImplementation((query) => {
                return mockGetNodePoolsByQuery(query);
            });
            const result = await nodePoolRepository.getProjectsAllowedNodePools(input);
            expect(result).toBeDefined();
            expect(spy).toHaveBeenCalledTimes(input.length);
            expect(result).toStrictEqual(mockGetProjectAllowedNodePoolsResult);
        });
        it('should throw error when nodepools cannot be extracted from the sourceId', async () => {
            const input = [{ id: 1, sourceId: 100 }];
            const mockGetProjectAllowedNodePoolsResult = {};
            const spy = jest.spyOn(adapterClass.prototype, 'getNodePoolsByQuery').mockImplementation(() => {
                return mockGetNodePoolsByQueryThrowError();
            });
            const result = await nodePoolRepository.getProjectsAllowedNodePools(input);
            expect(result).toBeDefined();
            expect(spy).toHaveBeenCalledTimes(input.length);
            expect(result).toStrictEqual(mockGetProjectAllowedNodePoolsResult);
        });
    });
    describe('getOrphanedNodes test cases', () => {
        it('should return empty arrays as response when no extracted nodes is an orphan node', async () => {
            const spy = jest.spyOn(adapterClass.prototype, 'getNodesByQuery').mockResolvedValueOnce([
                { id: 1, projectId: 50, isOnDemand: true, nodePoolId: 1001 },
                { id: 2, projectId: 60, isOnDemand: false, nodePoolId: 1003 }
            ]);
            const result = await nodePoolRepository.getOrphanedNodes();
            const mockExpectedResult = {
                orphanedOnDemandNodesByProjectId: [],
                orphanedNodePools: []
            };
            expect(spy).toHaveBeenCalledTimes(1);
            expect(spy).toHaveBeenCalledWith();
            expect(result).toStrictEqual(mockExpectedResult);
        });
        it('should return empty arrays as response when no extracted nodes is an on-demand-node', async () => {
            const spy = jest.spyOn(adapterClass.prototype, 'getNodesByQuery').mockResolvedValueOnce([
                { id: 1, projectId: 50, isOnDemand: false },
                { id: 2, projectId: 60, isOnDemand: false }
            ]);
            const result = await nodePoolRepository.getOrphanedNodes();
            const mockExpectedResult = {
                orphanedOnDemandNodesByProjectId: [],
                orphanedNodePools: []
            };
            expect(spy).toHaveBeenCalledTimes(1);
            expect(spy).toHaveBeenCalledWith();
            expect(result).toStrictEqual(mockExpectedResult);
        });
        it('should return list of orphanOnDemandNodesByProjectId and orphanOnDemandNodePools', async () => {
            const spy = jest.spyOn(adapterClass.prototype, 'getNodesByQuery').mockResolvedValueOnce([
                { id: 1, projectId: 50, isOnDemand: false },
                { id: 2, projectId: 60, isOnDemand: true, nodePoolId: 555 },
                { id: 3, projectId: 50, isOnDemand: true },
                { id: 4, projectId: 60, isOnDemand: true },
                { id: 5, projectId: 50, isOnDemand: true },
                { id: 6, projectId: 60, isOnDemand: true }
            ]);
            const result = await nodePoolRepository.getOrphanedNodes();
            const mockExpectedResult = {
                orphanedOnDemandNodesByProjectId: {
                    50: [
                        { id: 3, projectId: 50, isOnDemand: true },
                        { id: 5, projectId: 50, isOnDemand: true }
                    ],
                    60: [
                        { id: 4, projectId: 60, isOnDemand: true },
                        { id: 6, projectId: 60, isOnDemand: true }
                    ]
                },
                orphanedNodePools: [
                    {
                        name: 'orphaned#on-demand#nodepool',
                        projectId: 50,
                        isOrphaned: true,
                        isOnDemand: true
                    },
                    {
                        name: 'orphaned#on-demand#nodepool',
                        projectId: 60,
                        isOrphaned: true,
                        isOnDemand: true
                    }
                ]
            };
            expect(spy).toHaveBeenCalledTimes(1);
            expect(spy).toHaveBeenCalledWith();
            expect(result).toStrictEqual(mockExpectedResult);
        });
        it('should return empty responses when nodes cannot be extracted', async () => {
            const spy = jest.spyOn(adapterClass.prototype, 'getNodesByQuery').mockImplementation(() => {
                return mockErrorOutGetNodesByQuery();
            });
            const result = await nodePoolRepository.getOrphanedNodes();
            const mockExpectedResult = {
                orphanedOnDemandNodesByProjectId: [],
                orphanedNodePools: []
            };
            expect(result).toStrictEqual(mockExpectedResult);
            expect(spy).toHaveBeenCalledTimes(1);
            expect(spy).toHaveBeenCalledWith();
        });
    });
    describe('getNodes test cases', () => {
        it('should return all nodes when passed with an empty query', async () => {
            const spy = jest
                .spyOn(adapterClass.prototype, 'getNodesByQuery')
                .mockResolvedValueOnce([{ id: 1 }, { id: 2 }]);
            const result = await nodePoolRepository.getNodes();
            const mockExpectedResult = [{ id: 1 }, { id: 2 }];
            expect(result).toBeDefined();
            expect(spy).toHaveBeenCalledTimes(1);
            expect(result).toStrictEqual(mockExpectedResult);
            expect(spy).toHaveBeenCalledWith('');
        });
        it('should return all nodes matching the nodePoolId passed as the query', async () => {
            const query = 'nodePoolIds=100';
            const spy = jest.spyOn(adapterClass.prototype, 'getNodesByQuery').mockResolvedValueOnce([
                { id: 1, nodePoolId: 100 },
                { id: 2, nodePoolId: 100 }
            ]);
            const result = await nodePoolRepository.getNodes(query);
            const mockExpectedResult = [
                { id: 1, nodePoolId: 100 },
                { id: 2, nodePoolId: 100 }
            ];
            expect(result).toBeDefined();
            expect(spy).toHaveBeenCalledTimes(1);
            expect(result).toStrictEqual(mockExpectedResult);
            expect(spy).toHaveBeenCalledWith(query);
        });
    });
    describe('getAndFilterAvailableNodes test cases', () => {
        it('should return filteredNodes matching the nodePoolId', async () => {
            const nodePoolId = 10;
            const statusCodes = {
                queued: {
                    code: 4000,
                    name: 'queued',
                    group: 'status'
                },
                processing: {
                    code: 4001,
                    name: 'processing',
                    group: 'status'
                },
                success: {
                    code: 4002,
                    name: 'success',
                    group: 'status'
                },
                failure: {
                    code: 4003,
                    name: 'failure',
                    group: 'status'
                },
                error: {
                    code: 4004,
                    name: 'error',
                    group: 'status'
                },
                waiting: {
                    code: 4005,
                    name: 'waiting',
                    group: 'status'
                },
                cancelled: {
                    code: 4006,
                    name: 'cancelled',
                    group: 'status'
                },
                unstable: {
                    code: 4007,
                    name: 'unstable',
                    group: 'status'
                },
                skipped: {
                    code: 4008,
                    name: 'skipped',
                    group: 'status'
                },
                timeout: {
                    code: 4009,
                    name: 'timeout',
                    group: 'status'
                },
                stopped: {
                    code: 4010,
                    name: 'stopped',
                    group: 'status'
                },
                deleted: {
                    code: 4011,
                    name: 'deleted',
                    group: 'status'
                },
                cached: {
                    code: 4012,
                    name: 'cached',
                    group: 'status'
                },
                cancelling: {
                    code: 4013,
                    name: 'cancelling',
                    group: 'status'
                },
                timingOut: {
                    code: 4014,
                    name: 'timingOut',
                    group: 'status'
                },
                creating: {
                    code: 4015,
                    name: 'creating',
                    group: 'status'
                },
                ready: {
                    code: 4016,
                    name: 'ready',
                    group: 'status'
                },
                online: {
                    code: 4017,
                    name: 'online',
                    group: 'status'
                },
                offline: {
                    code: 4018,
                    name: 'offline',
                    group: 'status'
                },
                unhealthy: {
                    code: 4019,
                    name: 'unhealthy',
                    group: 'status'
                },
                onlineRequested: {
                    code: 4020,
                    name: 'onlineRequested',
                    group: 'status'
                },
                offlineRequested: {
                    code: 4021,
                    name: 'offlineRequested',
                    group: 'status'
                },
                pendingApproval: {
                    code: 4022,
                    name: 'pendingApproval',
                    group: 'status'
                }
            };
            const spy = jest.spyOn(adapterClass.prototype, 'getNodesByQuery').mockResolvedValueOnce([
                { id: 1, statusCode: 4001 },
                { id: 2, statusCode: 4002 },
                { id: 3, statusCode: 4003 },
                { id: 4, statusCode: 4004 },
                { id: 5, statusCode: 4005 },
                { id: 6, statusCode: 4006 }
            ]);
            const result = await nodePoolRepository.getAndFilterAvailableNodes(nodePoolId, statusCodes);
            const mockExpectedResult = [
                { id: 1, statusCode: 4001 },
                { id: 2, statusCode: 4002 },
                { id: 5, statusCode: 4005 }
            ];
            expect(result).toBeDefined();
            expect(spy).toHaveBeenCalledTimes(1);
            expect(result).toStrictEqual(mockExpectedResult);
        });
        it('should return empty list when no node is available node', async () => {
            const nodePoolId = 10;
            const statusCodes = {
                queued: {
                    code: 4000,
                    name: 'queued',
                    group: 'status'
                },
                processing: {
                    code: 4001,
                    name: 'processing',
                    group: 'status'
                },
                success: {
                    code: 4002,
                    name: 'success',
                    group: 'status'
                },
                failure: {
                    code: 4003,
                    name: 'failure',
                    group: 'status'
                },
                error: {
                    code: 4004,
                    name: 'error',
                    group: 'status'
                },
                waiting: {
                    code: 4005,
                    name: 'waiting',
                    group: 'status'
                },
                cancelled: {
                    code: 4006,
                    name: 'cancelled',
                    group: 'status'
                },
                unstable: {
                    code: 4007,
                    name: 'unstable',
                    group: 'status'
                },
                skipped: {
                    code: 4008,
                    name: 'skipped',
                    group: 'status'
                },
                timeout: {
                    code: 4009,
                    name: 'timeout',
                    group: 'status'
                },
                stopped: {
                    code: 4010,
                    name: 'stopped',
                    group: 'status'
                },
                deleted: {
                    code: 4011,
                    name: 'deleted',
                    group: 'status'
                },
                cached: {
                    code: 4012,
                    name: 'cached',
                    group: 'status'
                },
                cancelling: {
                    code: 4013,
                    name: 'cancelling',
                    group: 'status'
                },
                timingOut: {
                    code: 4014,
                    name: 'timingOut',
                    group: 'status'
                },
                creating: {
                    code: 4015,
                    name: 'creating',
                    group: 'status'
                },
                ready: {
                    code: 4016,
                    name: 'ready',
                    group: 'status'
                },
                online: {
                    code: 4017,
                    name: 'online',
                    group: 'status'
                },
                offline: {
                    code: 4018,
                    name: 'offline',
                    group: 'status'
                },
                unhealthy: {
                    code: 4019,
                    name: 'unhealthy',
                    group: 'status'
                },
                onlineRequested: {
                    code: 4020,
                    name: 'onlineRequested',
                    group: 'status'
                },
                offlineRequested: {
                    code: 4021,
                    name: 'offlineRequested',
                    group: 'status'
                },
                pendingApproval: {
                    code: 4022,
                    name: 'pendingApproval',
                    group: 'status'
                }
            };
            const spy = jest.spyOn(adapterClass.prototype, 'getNodesByQuery').mockResolvedValueOnce([
                { id: 1, statusCode: 4003 },
                { id: 2, statusCode: 4004 },
                { id: 3, statusCode: 4007 },
                { id: 4, statusCode: 4008 },
                { id: 5, statusCode: 4010 },
                { id: 6, statusCode: 4011 }
            ]);
            const result = await nodePoolRepository.getAndFilterAvailableNodes(nodePoolId, statusCodes);
            const mockExpectedResult = [];
            expect(result).toBeDefined();
            expect(spy).toHaveBeenCalledTimes(1);
            expect(result).toStrictEqual(mockExpectedResult);
        });
        it('should return all the nodes when all the nodes are available', async () => {
            const nodePoolId = 10;
            const statusCodes = {
                queued: {
                    code: 4000,
                    name: 'queued',
                    group: 'status'
                },
                processing: {
                    code: 4001,
                    name: 'processing',
                    group: 'status'
                },
                success: {
                    code: 4002,
                    name: 'success',
                    group: 'status'
                },
                failure: {
                    code: 4003,
                    name: 'failure',
                    group: 'status'
                },
                error: {
                    code: 4004,
                    name: 'error',
                    group: 'status'
                },
                waiting: {
                    code: 4005,
                    name: 'waiting',
                    group: 'status'
                },
                cancelled: {
                    code: 4006,
                    name: 'cancelled',
                    group: 'status'
                },
                unstable: {
                    code: 4007,
                    name: 'unstable',
                    group: 'status'
                },
                skipped: {
                    code: 4008,
                    name: 'skipped',
                    group: 'status'
                },
                timeout: {
                    code: 4009,
                    name: 'timeout',
                    group: 'status'
                },
                stopped: {
                    code: 4010,
                    name: 'stopped',
                    group: 'status'
                },
                deleted: {
                    code: 4011,
                    name: 'deleted',
                    group: 'status'
                },
                cached: {
                    code: 4012,
                    name: 'cached',
                    group: 'status'
                },
                cancelling: {
                    code: 4013,
                    name: 'cancelling',
                    group: 'status'
                },
                timingOut: {
                    code: 4014,
                    name: 'timingOut',
                    group: 'status'
                },
                creating: {
                    code: 4015,
                    name: 'creating',
                    group: 'status'
                },
                ready: {
                    code: 4016,
                    name: 'ready',
                    group: 'status'
                },
                online: {
                    code: 4017,
                    name: 'online',
                    group: 'status'
                },
                offline: {
                    code: 4018,
                    name: 'offline',
                    group: 'status'
                },
                unhealthy: {
                    code: 4019,
                    name: 'unhealthy',
                    group: 'status'
                },
                onlineRequested: {
                    code: 4020,
                    name: 'onlineRequested',
                    group: 'status'
                },
                offlineRequested: {
                    code: 4021,
                    name: 'offlineRequested',
                    group: 'status'
                },
                pendingApproval: {
                    code: 4022,
                    name: 'pendingApproval',
                    group: 'status'
                }
            };
            const spy = jest.spyOn(adapterClass.prototype, 'getNodesByQuery').mockResolvedValueOnce([
                { id: 1, statusCode: 4000 },
                { id: 2, statusCode: 4001 },
                { id: 3, statusCode: 4002 },
                { id: 4, statusCode: 4005 },
                { id: 5, statusCode: 4005 },
                { id: 6, statusCode: 4012 }
            ]);
            const result = await nodePoolRepository.getAndFilterAvailableNodes(nodePoolId, statusCodes);
            const mockExpectedResult = [
                { id: 1, statusCode: 4000 },
                { id: 2, statusCode: 4001 },
                { id: 3, statusCode: 4002 },
                { id: 4, statusCode: 4005 },
                { id: 5, statusCode: 4005 },
                { id: 6, statusCode: 4012 }
            ];
            expect(result).toBeDefined();
            expect(spy).toHaveBeenCalledTimes(1);
            expect(result).toStrictEqual(mockExpectedResult);
        });
        it('should return error when nodes cannot be extracted', async () => {
            const nodePoolId = 10;
            const statusCodes = {
                queued: {
                    code: 4000,
                    name: 'queued',
                    group: 'status'
                },
                processing: {
                    code: 4001,
                    name: 'processing',
                    group: 'status'
                },
                success: {
                    code: 4002,
                    name: 'success',
                    group: 'status'
                },
                failure: {
                    code: 4003,
                    name: 'failure',
                    group: 'status'
                },
                error: {
                    code: 4004,
                    name: 'error',
                    group: 'status'
                },
                waiting: {
                    code: 4005,
                    name: 'waiting',
                    group: 'status'
                },
                cancelled: {
                    code: 4006,
                    name: 'cancelled',
                    group: 'status'
                },
                unstable: {
                    code: 4007,
                    name: 'unstable',
                    group: 'status'
                },
                skipped: {
                    code: 4008,
                    name: 'skipped',
                    group: 'status'
                },
                timeout: {
                    code: 4009,
                    name: 'timeout',
                    group: 'status'
                },
                stopped: {
                    code: 4010,
                    name: 'stopped',
                    group: 'status'
                },
                deleted: {
                    code: 4011,
                    name: 'deleted',
                    group: 'status'
                },
                cached: {
                    code: 4012,
                    name: 'cached',
                    group: 'status'
                },
                cancelling: {
                    code: 4013,
                    name: 'cancelling',
                    group: 'status'
                },
                timingOut: {
                    code: 4014,
                    name: 'timingOut',
                    group: 'status'
                },
                creating: {
                    code: 4015,
                    name: 'creating',
                    group: 'status'
                },
                ready: {
                    code: 4016,
                    name: 'ready',
                    group: 'status'
                },
                online: {
                    code: 4017,
                    name: 'online',
                    group: 'status'
                },
                offline: {
                    code: 4018,
                    name: 'offline',
                    group: 'status'
                },
                unhealthy: {
                    code: 4019,
                    name: 'unhealthy',
                    group: 'status'
                },
                onlineRequested: {
                    code: 4020,
                    name: 'onlineRequested',
                    group: 'status'
                },
                offlineRequested: {
                    code: 4021,
                    name: 'offlineRequested',
                    group: 'status'
                },
                pendingApproval: {
                    code: 4022,
                    name: 'pendingApproval',
                    group: 'status'
                }
            };
            const spy = jest.spyOn(adapterClass.prototype, 'getNodesByQuery').mockRejectedValueOnce(new Error('error'));
            await expect(nodePoolRepository.getAndFilterAvailableNodes(nodePoolId, statusCodes)).rejects.toThrow('error');
            expect(spy).toHaveBeenCalledTimes(1);
        });
    });
    describe('getBuildPlaneMessagesCount test cases', () => {
        it('should return count of buildplane messages', async () => {
            const nodePoolId = 10;
            const statusCodes = {
                queued: {
                    code: 4000,
                    name: 'queued',
                    group: 'status'
                },
                processing: {
                    code: 4001,
                    name: 'processing',
                    group: 'status'
                },
                success: {
                    code: 4002,
                    name: 'success',
                    group: 'status'
                },
                failure: {
                    code: 4003,
                    name: 'failure',
                    group: 'status'
                },
                error: {
                    code: 4004,
                    name: 'error',
                    group: 'status'
                },
                waiting: {
                    code: 4005,
                    name: 'waiting',
                    group: 'status'
                },
                cancelled: {
                    code: 4006,
                    name: 'cancelled',
                    group: 'status'
                },
                unstable: {
                    code: 4007,
                    name: 'unstable',
                    group: 'status'
                },
                skipped: {
                    code: 4008,
                    name: 'skipped',
                    group: 'status'
                },
                timeout: {
                    code: 4009,
                    name: 'timeout',
                    group: 'status'
                },
                stopped: {
                    code: 4010,
                    name: 'stopped',
                    group: 'status'
                },
                deleted: {
                    code: 4011,
                    name: 'deleted',
                    group: 'status'
                },
                cached: {
                    code: 4012,
                    name: 'cached',
                    group: 'status'
                },
                cancelling: {
                    code: 4013,
                    name: 'cancelling',
                    group: 'status'
                },
                timingOut: {
                    code: 4014,
                    name: 'timingOut',
                    group: 'status'
                },
                creating: {
                    code: 4015,
                    name: 'creating',
                    group: 'status'
                },
                ready: {
                    code: 4016,
                    name: 'ready',
                    group: 'status'
                },
                online: {
                    code: 4017,
                    name: 'online',
                    group: 'status'
                },
                offline: {
                    code: 4018,
                    name: 'offline',
                    group: 'status'
                },
                unhealthy: {
                    code: 4019,
                    name: 'unhealthy',
                    group: 'status'
                },
                onlineRequested: {
                    code: 4020,
                    name: 'onlineRequested',
                    group: 'status'
                },
                offlineRequested: {
                    code: 4021,
                    name: 'offlineRequested',
                    group: 'status'
                },
                pendingApproval: {
                    code: 4022,
                    name: 'pendingApproval',
                    group: 'status'
                }
            };
            const spy = jest.spyOn(adapterClass.prototype, 'getBuildPlaneMessagesCountByQuery').mockResolvedValueOnce(1);
            const result = await nodePoolRepository.getBuildPlaneMessagesCount(statusCodes, nodePoolId);
            const mockExpectedResult = 1;
            expect(spy).toBeDefined();
            expect(result).toStrictEqual(mockExpectedResult);
            expect(spy).toHaveBeenCalledTimes(1);
        });
        it('should error out when adapter errors out', async () => {
            const nodePoolId = 10;
            const statusCodes = {
                queued: {
                    code: 4000,
                    name: 'queued',
                    group: 'status'
                },
                processing: {
                    code: 4001,
                    name: 'processing',
                    group: 'status'
                },
                success: {
                    code: 4002,
                    name: 'success',
                    group: 'status'
                },
                failure: {
                    code: 4003,
                    name: 'failure',
                    group: 'status'
                },
                error: {
                    code: 4004,
                    name: 'error',
                    group: 'status'
                },
                waiting: {
                    code: 4005,
                    name: 'waiting',
                    group: 'status'
                },
                cancelled: {
                    code: 4006,
                    name: 'cancelled',
                    group: 'status'
                },
                unstable: {
                    code: 4007,
                    name: 'unstable',
                    group: 'status'
                },
                skipped: {
                    code: 4008,
                    name: 'skipped',
                    group: 'status'
                },
                timeout: {
                    code: 4009,
                    name: 'timeout',
                    group: 'status'
                },
                stopped: {
                    code: 4010,
                    name: 'stopped',
                    group: 'status'
                },
                deleted: {
                    code: 4011,
                    name: 'deleted',
                    group: 'status'
                },
                cached: {
                    code: 4012,
                    name: 'cached',
                    group: 'status'
                },
                cancelling: {
                    code: 4013,
                    name: 'cancelling',
                    group: 'status'
                },
                timingOut: {
                    code: 4014,
                    name: 'timingOut',
                    group: 'status'
                },
                creating: {
                    code: 4015,
                    name: 'creating',
                    group: 'status'
                },
                ready: {
                    code: 4016,
                    name: 'ready',
                    group: 'status'
                },
                online: {
                    code: 4017,
                    name: 'online',
                    group: 'status'
                },
                offline: {
                    code: 4018,
                    name: 'offline',
                    group: 'status'
                },
                unhealthy: {
                    code: 4019,
                    name: 'unhealthy',
                    group: 'status'
                },
                onlineRequested: {
                    code: 4020,
                    name: 'onlineRequested',
                    group: 'status'
                },
                offlineRequested: {
                    code: 4021,
                    name: 'offlineRequested',
                    group: 'status'
                },
                pendingApproval: {
                    code: 4022,
                    name: 'pendingApproval',
                    group: 'status'
                }
            };
            const spy = jest
                .spyOn(adapterClass.prototype, 'getBuildPlaneMessagesCountByQuery')
                .mockRejectedValueOnce('error');
            await expect(nodePoolRepository.getBuildPlaneMessagesCount(statusCodes, nodePoolId)).rejects.toThrow('error');
            expect(spy).toBeDefined();
            expect(spy).toHaveBeenCalledTimes(1);
        });
    });
    describe('findAvailableNodeCount test cases', () => {
        const nodePoolRepository = new NodePoolRepository_1.NodePoolRepository();
        it('should return totalGroupsThatCanBeQueued, activeNodeCount, cachedNodeCount, cachedNodes in case of onDemandNodePool', () => {
            const isOnDemandNodePool = true;
            const processingGroups = [
                { projectId: 1, isExternal: false },
                { projectId: 2, isExternal: false },
                { projectId: 3, isExternal: false },
                { projectId: 4, isExternal: false }
            ];
            const useRunAdjacency = true;
            const buildPlaneMessagesCount = 1;
            const numberOfNodesNodePool = 4;
            const statusCodes = {
                queued: {
                    code: 4000,
                    name: 'queued',
                    group: 'status'
                },
                processing: {
                    code: 4001,
                    name: 'processing',
                    group: 'status'
                },
                success: {
                    code: 4002,
                    name: 'success',
                    group: 'status'
                },
                failure: {
                    code: 4003,
                    name: 'failure',
                    group: 'status'
                },
                error: {
                    code: 4004,
                    name: 'error',
                    group: 'status'
                },
                waiting: {
                    code: 4005,
                    name: 'waiting',
                    group: 'status'
                },
                cancelled: {
                    code: 4006,
                    name: 'cancelled',
                    group: 'status'
                },
                unstable: {
                    code: 4007,
                    name: 'unstable',
                    group: 'status'
                },
                skipped: {
                    code: 4008,
                    name: 'skipped',
                    group: 'status'
                },
                timeout: {
                    code: 4009,
                    name: 'timeout',
                    group: 'status'
                },
                stopped: {
                    code: 4010,
                    name: 'stopped',
                    group: 'status'
                },
                deleted: {
                    code: 4011,
                    name: 'deleted',
                    group: 'status'
                },
                cached: {
                    code: 4012,
                    name: 'cached',
                    group: 'status'
                },
                cancelling: {
                    code: 4013,
                    name: 'cancelling',
                    group: 'status'
                },
                timingOut: {
                    code: 4014,
                    name: 'timingOut',
                    group: 'status'
                },
                creating: {
                    code: 4015,
                    name: 'creating',
                    group: 'status'
                },
                ready: {
                    code: 4016,
                    name: 'ready',
                    group: 'status'
                },
                online: {
                    code: 4017,
                    name: 'online',
                    group: 'status'
                },
                offline: {
                    code: 4018,
                    name: 'offline',
                    group: 'status'
                },
                unhealthy: {
                    code: 4019,
                    name: 'unhealthy',
                    group: 'status'
                },
                onlineRequested: {
                    code: 4020,
                    name: 'onlineRequested',
                    group: 'status'
                },
                offlineRequested: {
                    code: 4021,
                    name: 'offlineRequested',
                    group: 'status'
                },
                pendingApproval: {
                    code: 4022,
                    name: 'pendingApproval',
                    group: 'status'
                }
            };
            const nodes = [
                { id: 1, statusCode: statusCodes.success.code },
                { id: 2, statusCode: statusCodes.cached.code }
            ];
            const isCacheEnabledNodePool = true;
            const result = nodePoolRepository.findAvailableNodeCount(isOnDemandNodePool, processingGroups, useRunAdjacency, buildPlaneMessagesCount, numberOfNodesNodePool, statusCodes, nodes, isCacheEnabledNodePool);
            const mockExpectedResult = {
                totalGroupsThatCanBeQueued: 3,
                cachedNodes: [{ id: 2, statusCode: 4012 }],
                activeNodesCount: 1,
                cachedNodesCount: 1
            };
            console.log(result);
            expect(result).toStrictEqual(mockExpectedResult);
        });
        it('should return totalGroupsThatCanBeQueued in case of onDemandNodePool to be false', () => {
            const isOnDemandNodePool = false;
            const processingGroups = [
                { projectId: 1, isExternal: false },
                { projectId: 2, isExternal: false },
                { projectId: 3, isExternal: false },
                { projectId: 4, isExternal: false }
            ];
            const useRunAdjacency = true;
            const buildPlaneMessagesCount = 1;
            const numberOfNodesNodePool = 4;
            const statusCodes = {
                queued: {
                    code: 4000,
                    name: 'queued',
                    group: 'status'
                },
                processing: {
                    code: 4001,
                    name: 'processing',
                    group: 'status'
                },
                success: {
                    code: 4002,
                    name: 'success',
                    group: 'status'
                },
                failure: {
                    code: 4003,
                    name: 'failure',
                    group: 'status'
                },
                error: {
                    code: 4004,
                    name: 'error',
                    group: 'status'
                },
                waiting: {
                    code: 4005,
                    name: 'waiting',
                    group: 'status'
                },
                cancelled: {
                    code: 4006,
                    name: 'cancelled',
                    group: 'status'
                },
                unstable: {
                    code: 4007,
                    name: 'unstable',
                    group: 'status'
                },
                skipped: {
                    code: 4008,
                    name: 'skipped',
                    group: 'status'
                },
                timeout: {
                    code: 4009,
                    name: 'timeout',
                    group: 'status'
                },
                stopped: {
                    code: 4010,
                    name: 'stopped',
                    group: 'status'
                },
                deleted: {
                    code: 4011,
                    name: 'deleted',
                    group: 'status'
                },
                cached: {
                    code: 4012,
                    name: 'cached',
                    group: 'status'
                },
                cancelling: {
                    code: 4013,
                    name: 'cancelling',
                    group: 'status'
                },
                timingOut: {
                    code: 4014,
                    name: 'timingOut',
                    group: 'status'
                },
                creating: {
                    code: 4015,
                    name: 'creating',
                    group: 'status'
                },
                ready: {
                    code: 4016,
                    name: 'ready',
                    group: 'status'
                },
                online: {
                    code: 4017,
                    name: 'online',
                    group: 'status'
                },
                offline: {
                    code: 4018,
                    name: 'offline',
                    group: 'status'
                },
                unhealthy: {
                    code: 4019,
                    name: 'unhealthy',
                    group: 'status'
                },
                onlineRequested: {
                    code: 4020,
                    name: 'onlineRequested',
                    group: 'status'
                },
                offlineRequested: {
                    code: 4021,
                    name: 'offlineRequested',
                    group: 'status'
                },
                pendingApproval: {
                    code: 4022,
                    name: 'pendingApproval',
                    group: 'status'
                }
            };
            const nodes = [
                { id: 1, statusCode: statusCodes.success.code },
                { id: 2, statusCode: statusCodes.cached.code }
            ];
            const isCacheEnabledNodePool = true;
            const result = nodePoolRepository.findAvailableNodeCount(isOnDemandNodePool, processingGroups, useRunAdjacency, buildPlaneMessagesCount, numberOfNodesNodePool, statusCodes, nodes, isCacheEnabledNodePool);
            const mockExpectedResult = 2;
            console.log(result);
            expect(result.totalGroupsThatCanBeQueued).toStrictEqual(mockExpectedResult);
        });
    });
});
//# sourceMappingURL=NodePoolRepository.test.js.map