"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SchedularRootAggregate = void 0;
const node_util_1 = require("node:util");
const logger_1 = require("@jfrog-pipelines/logger");
const configuration_1 = require("@jfrog-pipelines/configuration");
const license_1 = require("./license");
const dag_1 = require("./dag");
const Queuer_1 = require("./queuer/Queuer");
const ConsoleLogRepository_1 = require("./queuer/repository/ConsoleLogRepository");
var STEP_TRIGGER_STATUS;
(function (STEP_TRIGGER_STATUS) {
    STEP_TRIGGER_STATUS["SUCCESS"] = "success";
    STEP_TRIGGER_STATUS["FAILURE"] = "failure";
    STEP_TRIGGER_STATUS["NOT_STARTED"] = "not-started";
})(STEP_TRIGGER_STATUS || (STEP_TRIGGER_STATUS = {}));
class SchedularRootAggregate {
    constructor(lock, sessionId) {
        this.lock = lock;
        this.sessionId = sessionId;
        this.dagRepository = new dag_1.DagVertexDataRepository();
        this.queuer = new Queuer_1.Queuer();
        this.consoleLogRepository = new ConsoleLogRepository_1.ConsoleLogRepository();
        this.logger = logger_1.LoggerFactory.getLogger(`${sessionId}:JobSchedularRootAggregate`);
    }
    async process() {
        const startTime = Date.now();
        let stepTriggerStatus = STEP_TRIGGER_STATUS.NOT_STARTED;
        try {
            const dagRecords = await new dag_1.DependencyModeDagRecordRepository(new dag_1.ChronologicalModeDagRecordRepository(this.dagRepository)).fetch();
            if (dagRecords.length === 0) {
                this.logger.info(`No valid dag records found. Exiting`);
                return;
            }
            this.logger.info(`${dagRecords.length} dag records found.`);
            const properties = configuration_1.ConfigFactory.config;
            const concurrentQuota = await license_1.ConcurrentQuota.create(properties);
            const availableNodeCount = await concurrentQuota.getCurrentExecutionSnapshot();
            if (availableNodeCount === 0) {
                this.logger.info(`Bailing out because maximum concurrent execution based on license has reached.`);
                const stepIdsSet = new Set(dagRecords.map((dagVertex) => dagVertex.stepId));
                const pipelineId = dagRecords[0].pipelineId;
                await this.consoleLogRepository.publishLicenseBreachedLogs([...stepIdsSet.values()], pipelineId);
                return;
            }
            const stepsByStepId = await this.fetchStepsByStepId(dagRecords);
            const stepletsByStepId = await this.dagRepository.getStepletByStepIds([...stepsByStepId.keys()]);
            const nodePoolsByStepletId = await this.dagRepository.getNodePoolsBySteplets([...stepletsByStepId.values()].flat());
            this.logger.info(`Processing ${[...stepsByStepId.keys()].length} steps`);
            const dagAggregate = new dag_1.DagAggregate(dagRecords, stepsByStepId, stepletsByStepId);
            const stepAffinityList = dagAggregate.getStepAffinityList(nodePoolsByStepletId);
            const stepAffinityIterable = new dag_1.StepAffinityIterable(stepAffinityList, concurrentQuota);
            this.logger.info(`Queueing ${stepAffinityIterable.size()} stepAffinitys`);
            await this.queuer.queue(stepAffinityIterable);
            stepTriggerStatus = STEP_TRIGGER_STATUS.SUCCESS;
        }
        catch (error) {
            this.logger.warn(`Encountered an unexpected error:`, (0, node_util_1.inspect)(error));
            stepTriggerStatus = STEP_TRIGGER_STATUS.FAILURE;
        }
        finally {
            await this.lock.releaseLock();
            const stopTime = Date.now();
            const timeDiff = stopTime - startTime;
            const eventData = {
                timestamp: new Date(stopTime).toISOString(),
                domain: 'stepService',
                eventType: 'steptriggeriteration',
                status: stepTriggerStatus,
                durationMillis: timeDiff.valueOf(),
                jfrogPipelinesSessionId: this.sessionId
            };
            logger_1.LoggerFactory.logEvent(eventData);
        }
    }
    async fetchStepsByStepId(dagRecords) {
        const stepIdsSet = new Set(dagRecords.map((record) => record.stepId));
        const stepsByStepId = await this.dagRepository.getStepsByStepIds([...stepIdsSet.values()]);
        return stepsByStepId;
    }
}
exports.SchedularRootAggregate = SchedularRootAggregate;
//# sourceMappingURL=SchedularRootAggregate.js.map