"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DagAggregate = void 0;
const node_util_1 = require("node:util");
const logger_1 = require("@jfrog-pipelines/logger");
const DagVertex_1 = require("./models/DagVertex");
const models_1 = require("./models");
const vo_1 = require("./models/vo");
class DagAggregate {
    constructor(dagRecords, stepMap, stepletMap) {
        this.logger = logger_1.LoggerFactory.getLogger('DagAggregate');
        this.dagVertexList = this.buildDag(dagRecords, stepMap, stepletMap);
    }
    getStepAffinityList(nodePoolByStepletId) {
        const dagMap = new Map();
        for (const record of this.dagVertexList) {
            const dagUidVo = new vo_1.DagId(record.getRunId(), record.getAffinityGroup());
            const key = dagUidVo.getId();
            const existingRecords = [...(dagMap.get(key) ?? []), record];
            dagMap.set(key, existingRecords);
        }
        const stepAffinityList = [];
        for (const [dagUid, dagList] of dagMap) {
            if (dagList.length === 0) {
                this.logger.warn(`empty dag records found for dagId: ${dagUid}`);
                continue;
            }
            const anyDag = dagList[0];
            const runId = anyDag.getRunId();
            const affinityGroup = anyDag.getAffinityGroup();
            const steplets = anyDag.getSteps().getSteplets();
            if (steplets.length === 0) {
                this.logger.warn(`no steplets found for dagId: ${dagUid}`);
                continue;
            }
            const anySteplet = steplets[0];
            const nodePool = nodePoolByStepletId?.get(anySteplet.getId()) ?? null;
            try {
                const stepsInOrder = this.getStepsInSortedOrder(dagList);
                const stepAffinity = models_1.StepAffinity.create(runId, affinityGroup, dagList, nodePool, stepsInOrder);
                if (stepAffinity) {
                    stepAffinityList.push(stepAffinity);
                }
            }
            catch (error) {
                this.logger.error((0, node_util_1.inspect)(error));
            }
        }
        return stepAffinityList;
    }
    buildDag(dagRecords, stepMap, stepletMap) {
        const dagVertexList = [];
        for (const record of dagRecords) {
            const stepId = record.stepId;
            const step = stepMap.get(stepId);
            if (step === undefined) {
                this.logger.warn(`No step with stepId: ${stepId} found for dagRecord with id ${record.id}`);
                continue;
            }
            const stepletAffinityGroupMap = this.getStepletsGroupedByAffinityGroup(stepletMap, stepId);
            if (stepletAffinityGroupMap === null) {
                this.logger.warn(`No steplets found for stepId: ${stepId} and dagRecord with Id ${record.id}`);
                continue;
            }
            for (const steplets of stepletAffinityGroupMap.values()) {
                const stepletList = [];
                for (const steplet of steplets) {
                    const stepletModel = models_1.Steplet.create(steplet);
                    stepletList.push(stepletModel);
                }
                const stepModel = models_1.Step.create(step, stepletList);
                const dagVertex = DagVertex_1.DagVertex.create(record, stepModel);
                dagVertexList.push(dagVertex);
            }
        }
        return dagVertexList;
    }
    getStepletsGroupedByAffinityGroup(stepletMap, stepId) {
        const steplets = stepletMap.get(stepId);
        if (steplets === undefined) {
            return null;
        }
        const stepletAffinityGroupMap = new Map();
        for (const steplet of steplets) {
            const stepletList = [...(stepletAffinityGroupMap.get(steplet.affinityGroup) ?? []), steplet];
            stepletAffinityGroupMap.set(steplet.affinityGroup, stepletList);
        }
        return stepletAffinityGroupMap;
    }
    getStepsInSortedOrder(dagList) {
        const stepIdsSet = new Set(dagList.map((dagVertex) => dagVertex.getStepId()));
        const stepCount = stepIdsSet.size;
        if (stepCount === 1) {
            dagList[0].getSteps();
        }
        const filteredDagList = dagList.filter((dagVertex) => dagVertex.getDependentEntityType() !== -1);
        let stepCoRelationMap = this.getStepCoRelationMap(filteredDagList, stepIdsSet);
        const stepsInOrder = [];
        for (let iteration = 1; iteration <= stepCount; iteration += 1) {
            const step = this.getStepWithNoDependency(stepCoRelationMap, dagList);
            if (step === null) {
                continue;
            }
            stepCoRelationMap = this.removeDependency(stepCoRelationMap, step.getId());
            stepsInOrder.push(step);
        }
        return stepsInOrder;
    }
    getStepCoRelationMap(filteredDagList, stepIdsSet) {
        const stepCoRelationMap = new Map();
        for (const stepId of stepIdsSet) {
            stepCoRelationMap.set(stepId, [-1]);
        }
        for (const dagVertex of filteredDagList) {
            const stepId = dagVertex.getStepId();
            const dependentStepId = dagVertex.getDependentEntityId();
            if (dependentStepId === null || dagVertex.getDependentEntityType() !== 0) {
                continue;
            }
            const existingDependentStepIds = stepCoRelationMap.get(stepId) ?? [];
            existingDependentStepIds.push(dependentStepId);
            stepCoRelationMap.set(stepId, existingDependentStepIds);
        }
        return stepCoRelationMap;
    }
    getStepWithNoDependency(stepCoRelationMap, dagList) {
        for (const [stepId, dependentStepIdList] of stepCoRelationMap) {
            if (dependentStepIdList.length === 1) {
                return this.getStepFromStepId(stepId, dagList);
            }
        }
        throw new Error(`Cyclic loop found. Skipping`);
    }
    getStepFromStepId(stepId, dagList) {
        const steps = [...dagList.map((dagVertex) => dagVertex.getSteps())];
        return steps.find((step) => step.getId() === stepId) ?? null;
    }
    removeDependency(stepCoRelationMap, removableStepId) {
        const resultantStepCoRelationMap = new Map();
        for (const [stepId, dependentStepId] of stepCoRelationMap) {
            if (stepId !== removableStepId) {
                const filteredDependentEntityList = dependentStepId.filter((dependentId) => dependentId != removableStepId);
                resultantStepCoRelationMap.set(stepId, filteredDependentEntityList);
            }
        }
        return resultantStepCoRelationMap;
    }
}
exports.DagAggregate = DagAggregate;
//# sourceMappingURL=DagAggregate.js.map