"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DagVertexDataRepository = void 0;
const node_util_1 = require("node:util");
const data_access_layer_1 = require("@jfrog-pipelines/data-access-layer");
const logger_1 = require("@jfrog-pipelines/logger");
const utility_1 = require("@jfrog-pipelines/utility");
const driven_1 = require("../../../../driven");
const models_1 = require("../models");
const QUERY_BATCH_SIZE = 100;
const CALL_BATCH_SIZE = 50;
class DagVertexDataRepository {
    constructor() {
        this.logger = logger_1.LoggerFactory.getLogger('DagRepository');
        this.dagDao = new driven_1.DagDao();
        this.stepDao = new driven_1.StepDao();
        this.stepletDao = new driven_1.StepletDao();
        this.projectDao = new driven_1.ProjectDao();
        this.nodePoolDao = new driven_1.NodePoolDao();
    }
    async fetch() {
        try {
            const records = await this.dagDao.getTriggerableSteps();
            if (records.length === 0) {
                return [];
            }
            return records;
        }
        catch (error) {
            this.logger.warn(`Cannot get dag records due to error: ${(0, node_util_1.inspect)(error)}`);
            return [];
        }
    }
    async getStepsByStepIds(stepIds) {
        const stepMap = new Map();
        try {
            const chunks = (0, utility_1.chunk)(stepIds, QUERY_BATCH_SIZE);
            const steps = [];
            await (0, utility_1.eachLimit)(chunks, CALL_BATCH_SIZE, async (stepIds) => {
                const stepQuery = new data_access_layer_1.SequelizeQueryBuilder({})
                    .whereWithOperator('id', stepIds, '$in')
                    .attributes(['id', 'affinityGroup', 'configPropertyBag', 'requiresApproval', 'pipelineSourceId']);
                const batchSteps = await this.stepDao.findAll(stepQuery.build());
                steps.push(...batchSteps);
            });
            for (const step of steps) {
                stepMap.set(step.id, step);
            }
            return stepMap;
        }
        catch (error) {
            this.logger.warn(`Cannot fetch StepsByStepId due to error: ${(0, node_util_1.inspect)(error)}`);
            return stepMap;
        }
    }
    async getStepletByStepIds(stepIds) {
        const stepletsByStepId = new Map();
        try {
            const chunks = (0, utility_1.chunk)(stepIds, QUERY_BATCH_SIZE);
            const steplets = [];
            await (0, utility_1.eachLimit)(chunks, CALL_BATCH_SIZE, async (stepIds) => {
                const stepletQuery = new data_access_layer_1.SequelizeQueryBuilder({})
                    .whereWithOperator('stepId', stepIds, '$in')
                    .attributes([
                    'id',
                    'priority',
                    'configPropertyBag',
                    'stepId',
                    'runtimePropertyBag',
                    'triggeredAt',
                    'pipelineId',
                    'pipelineSourceId'
                ]);
                const batchSteps = await this.stepletDao.findAll(stepletQuery.build());
                steplets.push(...batchSteps);
            });
            for (const steplet of steplets) {
                const stepletList = [...(stepletsByStepId.get(steplet.stepId) ?? []), steplet];
                stepletsByStepId.set(steplet.stepId, stepletList);
            }
            return stepletsByStepId;
        }
        catch (error) {
            this.logger.warn(`Cannot fetch StepletsByStepId due to error: ${(0, node_util_1.inspect)(error)}`);
            return stepletsByStepId;
        }
    }
    async getNodePoolsBySteplets(steplets) {
        try {
            const stepletsWithNodePools = steplets.filter((steplet) => steplet.configPropertyBag.nodePool);
            const nodePoolNamesSet = new Set(stepletsWithNodePools.map((steplet) => steplet.configPropertyBag.nodePool));
            const nodePoolByStepletId = new Map();
            const nodePools = await this.getNodePoolsByName([...nodePoolNamesSet.values()]);
            const projectIdSet = new Set(nodePools.map((nodePool) => nodePool.projectId));
            const projectIds = [...projectIdSet.values()];
            const projectMap = await this.getProjectByIds(projectIds);
            const nodePoolVoList = [];
            for (const nodePool of nodePools) {
                const project = projectMap.get(nodePool.projectId);
                if (project === undefined) {
                    this.logger.warn(`Skipping, a nodepool ${nodePool.name}, that doesn't have project for id ${nodePool.projectId} `);
                    continue;
                }
                const nodePoolModel = models_1.NodePool.create(nodePool, project);
                nodePoolVoList.push(nodePoolModel);
            }
            for (const steplet of steplets) {
                const nodePool = nodePoolVoList.find((nodePool) => nodePool.getName() === steplet.configPropertyBag.nodePool);
                nodePoolByStepletId.set(steplet.id, nodePool);
            }
            return nodePoolByStepletId;
        }
        catch (error) {
            this.logger.warn(`Cannot prepare nodepool value object due to error: ${(0, node_util_1.inspect)(error)}`);
            return null;
        }
    }
    getNodePoolsByName(nodePoolNames) {
        const query = new data_access_layer_1.SequelizeQueryBuilder({})
            .whereWithOperator('name', nodePoolNames, '$in')
            .attributes([
            'id',
            'projectId',
            'name',
            'numberOfNodes',
            'isOnDemand',
            'projectIntegrationId',
            'providerMetadataPropertyBag'
        ]);
        return this.nodePoolDao.findAll(query.build());
    }
    async getProjectByIds(projectIds) {
        const query = new data_access_layer_1.SequelizeQueryBuilder({})
            .whereWithOperator('id', projectIds, '$in')
            .attributes(['id', 'name', 'sourceId']);
        const projects = await this.projectDao.findAll(query.build());
        const projectMap = new Map();
        for (const project of projects) {
            projectMap.set(project.id, project);
        }
        return projectMap;
    }
}
exports.DagVertexDataRepository = DagVertexDataRepository;
//# sourceMappingURL=DagVertexDataRepository.js.map