"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChronologicalModeDagRecordRepository = void 0;
const data_access_layer_1 = require("@jfrog-pipelines/data-access-layer");
const metadata_1 = require("@jfrog-pipelines/metadata");
const driven_1 = require("../../../../../driven");
const NON_TERMINAL_STATUS = [metadata_1.CREATING, metadata_1.READY, metadata_1.WAITING, metadata_1.PROCESSING, metadata_1.TIMINGOUT, metadata_1.CANCELLING];
class ChronologicalModeDagRecordRepository {
    constructor(delegate) {
        this.delegate = delegate;
        this.runDao = new driven_1.RunDao();
        this.stepDao = new driven_1.StepDao();
    }
    async fetch() {
        const dagRecords = await this.delegate.fetch();
        const filteredListAfterChronologicalRun = await this.filterByChronologicalRun(dagRecords);
        const filteredListAfterChronologicalStep = await this.filterByChronologicalStep(filteredListAfterChronologicalRun);
        return filteredListAfterChronologicalStep;
    }
    async filterByChronologicalRun(dagRecords) {
        const chronologicalPipelineIdsSet = new Set();
        for (const record of dagRecords) {
            if (record.isChronologicalRun) {
                chronologicalPipelineIdsSet.add(record.pipelineId);
            }
        }
        if (chronologicalPipelineIdsSet.size === 0) {
            return dagRecords;
        }
        const pipelineIds = [...chronologicalPipelineIdsSet];
        const builder = new data_access_layer_1.SequelizeQueryBuilder({});
        const query = builder
            .where('statusCode', NON_TERMINAL_STATUS)
            .where('pipelineId', pipelineIds)
            .attributes(['id', 'pipelineId', 'runNumber'])
            .sort()
            .build();
        const incompleteRuns = await this.runDao.findAll(query);
        const incompleteRunsByPipelineId = new Map();
        for (const run of incompleteRuns) {
            const runs = incompleteRunsByPipelineId.get(run.pipelineId) ?? [];
            runs.push(run);
            incompleteRunsByPipelineId.set(run.pipelineId, runs);
        }
        const rejectedRunIds = new Set();
        for (const pipelineId of pipelineIds) {
            const runs = incompleteRunsByPipelineId.get(pipelineId) ?? [];
            if (runs.length === 0) {
                continue;
            }
            runs.sort((a, b) => a.runNumber - b.runNumber);
            runs.shift();
            runs.map((run) => rejectedRunIds.add(run.id));
        }
        return dagRecords.filter((dag) => {
            return !rejectedRunIds.has(dag.runId);
        });
    }
    async filterByChronologicalStep(dagRecords) {
        const chronologicalStepIdsSet = new Set();
        for (const record of dagRecords) {
            if (record.isChronologicalStep) {
                chronologicalStepIdsSet.add(record.chronologicalPipelineStepId);
            }
        }
        if (chronologicalStepIdsSet.size === 0) {
            return dagRecords;
        }
        const pipelineIds = [...chronologicalStepIdsSet];
        const chronologicalStepsByPipelineStepIds = new Map();
        const builder = new data_access_layer_1.SequelizeQueryBuilder({});
        const query = builder
            .where('statusCode', NON_TERMINAL_STATUS)
            .where('pipelineStepId', pipelineIds)
            .attributes(['id', 'pipelineId', 'pipelineStepId', 'affinityGroup', 'runId'])
            .sort()
            .build();
        const incompleteSteps = await this.stepDao.findAll(query);
        for (const step of incompleteSteps) {
            const steps = chronologicalStepsByPipelineStepIds.get(step.pipelineId) ?? [];
            steps.push(step);
            chronologicalStepsByPipelineStepIds.set(step.pipelineId, steps);
        }
        const rejectableAffinityGroups = new Set();
        const rejectableRunIds = new Set();
        const rejectableStepIds = new Set();
        for (const pipelineId of pipelineIds) {
            const steps = chronologicalStepsByPipelineStepIds.get(pipelineId) ?? [];
            if (steps.length === 0) {
                continue;
            }
            steps.sort((a, b) => {
                return a.runId - b.runId;
            });
            steps.shift();
            steps.map((step) => {
                rejectableStepIds.add(step.id);
                rejectableAffinityGroups.add(step.affinityGroup);
                rejectableRunIds.add(step.runId);
            });
        }
        return dagRecords.filter((dag) => {
            return !(rejectableRunIds.has(dag.runId) &&
                (rejectableAffinityGroups.has(dag.affinityGroup) || rejectableStepIds.has(dag.stepId)));
        });
    }
}
exports.ChronologicalModeDagRecordRepository = ChronologicalModeDagRecordRepository;
//# sourceMappingURL=ChronologicalModeDagRecordRepository.js.map