"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DependencyModeDagRecordRepository = void 0;
const node_util_1 = require("node:util");
const data_access_layer_1 = require("@jfrog-pipelines/data-access-layer");
const metadata_1 = require("@jfrog-pipelines/metadata");
const utility_1 = require("@jfrog-pipelines/utility");
const logger_1 = require("@jfrog-pipelines/logger");
const driven_1 = require("../../../../../driven");
const NON_TERMINAL_STATUS = [metadata_1.CREATING, metadata_1.READY, metadata_1.WAITING, metadata_1.PROCESSING, metadata_1.TIMINGOUT, metadata_1.CANCELLING];
const CALL_BATCH_SIZE = 5;
class DependencyModeDagRecordRepository {
    constructor(delegate) {
        this.delegate = delegate;
        this.logger = logger_1.LoggerFactory.getLogger('DependencyModeFilterRepository');
        this.runDao = new driven_1.RunDao();
        this.pipelineDao = new driven_1.PipelineDao();
    }
    async fetch() {
        const dagRecords = await this.delegate.fetch();
        const parentCompletePipelineIdsSet = new Set();
        const childCompletePipelineIdsSet = new Set();
        const parentSuccessPipelineIdsSet = new Set();
        for (const dag of dagRecords) {
            if (dag.dependentEntityType === 3) {
                parentCompletePipelineIdsSet.add(Number(dag.dependentEntityId));
            }
            else if (dag.dependentEntityType === 4) {
                parentSuccessPipelineIdsSet.add(Number(dag.dependentEntityId));
            }
            else if (dag.dependentEntityType === 5) {
                childCompletePipelineIdsSet.add(Number(dag.dependentEntityId));
            }
        }
        const allPipelineIdsSet = new Set([
            ...parentCompletePipelineIdsSet.values(),
            ...childCompletePipelineIdsSet.values(),
            ...parentSuccessPipelineIdsSet.values()
        ]);
        if (allPipelineIdsSet.size === 0) {
            return dagRecords;
        }
        const incompleteRuns = await this.getRunsByPipeline([...allPipelineIdsSet.values()], NON_TERMINAL_STATUS);
        const incompleteRunsByPipelineId = new Map();
        for (const run of incompleteRuns) {
            const pipelineId = run.pipelineId;
            const runs = incompleteRunsByPipelineId.get(pipelineId) ?? [];
            runs.push(run);
            incompleteRunsByPipelineId.set(pipelineId, runs);
        }
        const checkForParentSuccess = new Set();
        let dependencyModePipelinesUnresolved = new Set();
        for (const pipelineId of allPipelineIdsSet) {
            const incompleteRuns = incompleteRunsByPipelineId.get(pipelineId) ?? [];
            if (!(0, utility_1.isEmpty)(incompleteRuns)) {
                dependencyModePipelinesUnresolved.add(pipelineId);
            }
            else {
                if (parentSuccessPipelineIdsSet.has(pipelineId)) {
                    checkForParentSuccess.add(pipelineId);
                }
            }
        }
        if (checkForParentSuccess.size > 0) {
            const pipelineIds = [...checkForParentSuccess.values()];
            const unresolvedPipelines = await this.checkForParentSuccess(pipelineIds);
            dependencyModePipelinesUnresolved = new Set([
                ...unresolvedPipelines.values(),
                ...dependencyModePipelinesUnresolved.values()
            ]);
        }
        if (dependencyModePipelinesUnresolved.size === 0) {
            return dagRecords;
        }
        const waitingPipelines = new Set();
        const dependencyModeEntityTypes = new Set([
            5,
            3,
            4
        ]);
        for (const dag of dagRecords) {
            if (dependencyModeEntityTypes.has(dag.dependentEntityType) &&
                dependencyModePipelinesUnresolved.has(Number(dag.dependentEntityId))) {
                waitingPipelines.add(dag.pipelineId);
            }
        }
        return dagRecords.filter((dag) => {
            return !waitingPipelines.has(dag.pipelineId);
        });
    }
    async checkForParentSuccess(pipelineIds) {
        const unresolvedPipelines = new Set();
        const pipelines = await this.getPipelines(pipelineIds);
        for (const pipeline of pipelines) {
            if (!(0, utility_1.isEmpty)(pipeline) && pipeline.latestCompletedRunId != pipeline.latestSuccessfulRunId) {
                unresolvedPipelines.add(pipeline.id);
            }
        }
        return unresolvedPipelines;
    }
    async getRunsByPipeline(pipelineIds, statusCodes) {
        if (pipelineIds.length === 0) {
            return [];
        }
        try {
            const runs = [];
            const chunks = (0, utility_1.chunk)(pipelineIds, CALL_BATCH_SIZE);
            for (const batch of chunks) {
                const query = new data_access_layer_1.SequelizeQueryBuilder({})
                    .whereWithOperator('pipelineId', batch, '$in')
                    .whereWithOperator('statusCode', statusCodes, '$in')
                    .attributes(['id', 'pipelineId']);
                const runBatchResults = await this.runDao.findAll(query.build());
                runs.push(...runBatchResults);
            }
            runs.sort((a, b) => a.id - b.id);
            return runs;
        }
        catch (error) {
            this.logger.warn(`Cannot get runs due to error: ${(0, node_util_1.inspect)(error)}`);
            return [];
        }
    }
    async getPipelines(pipelineIds) {
        if (pipelineIds.length === 0) {
            return [];
        }
        try {
            const pipelines = [];
            const chunks = (0, utility_1.chunk)(pipelineIds, CALL_BATCH_SIZE);
            for (const batch of chunks) {
                const query = new data_access_layer_1.SequelizeQueryBuilder({})
                    .where('id', batch)
                    .attributes(['id', 'latestSuccessfulRunId', 'latestCompletedRunId']);
                const pipelineBatchResults = await this.pipelineDao.findAll(query.build());
                pipelines.push(...pipelineBatchResults);
            }
            return pipelines;
        }
        catch (error) {
            this.logger.warn(`Cannot get pipelines due to error: ${(0, node_util_1.inspect)(error)}`);
            return [];
        }
    }
}
exports.DependencyModeDagRecordRepository = DependencyModeDagRecordRepository;
//# sourceMappingURL=DependencyModeDagRecordRepository.js.map