"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConsoleLogRepository = void 0;
const node_util_1 = require("node:util");
const logger_1 = require("@jfrog-pipelines/logger");
const utility_1 = require("@jfrog-pipelines/utility");
const driven_1 = require("../../../../driven");
const shared_1 = require("../../../../shared");
const ConsoleMessage_1 = require("../models/constants/ConsoleMessage");
const THOUSAND = 1000;
class ConsoleLogRepository {
    constructor() {
        this.logger = logger_1.LoggerFactory.getLogger(`ConsoleLogManager`);
        this.logServiceQueueClient = new driven_1.LogServiceQueueClient();
    }
    async publishNodePoolQueuerQueuedLogs(stepletIds, pipelineId, nodePoolName = '') {
        const grpMessage = 'Queueing Step';
        const cmdMessage = 'Info';
        const message = ConsoleMessage_1.ConsoleMessage.getNodePoolQueueMessage(nodePoolName);
        await this.publishLogs(pipelineId, true, grpMessage, cmdMessage, message, stepletIds);
    }
    async publishExternalQueuerQueuedLogs(stepletIds, pipelineId) {
        const grpMessage = 'Queueing Step';
        const cmdMessage = 'Info';
        const message = ConsoleMessage_1.ConsoleMessage.getExternalQueueMessage();
        await this.publishLogs(pipelineId, true, grpMessage, cmdMessage, message, stepletIds);
    }
    async publishApprovalRequestLogs(stepletIds, pipelineId) {
        const grpMessage = 'Waiting for approvals';
        const cmdMessage = 'Awaiting approval from required approvers.';
        const message = `\nThe current step won't start until the required approvers approve this step.`;
        await this.publishLogs(pipelineId, true, grpMessage, cmdMessage, message, stepletIds);
    }
    async publishSkipLogsWhenNoNode(stepletIds, pipelineId) {
        const grpMessage = 'Queuing Step';
        const cmdMessage = 'Info';
        const message = `Skipping step as no node found. Please try adding a node to initiate execution.`;
        await this.publishLogs(pipelineId, false, grpMessage, cmdMessage, message, stepletIds);
    }
    async publishSkipLogsWhenStepNotEligible(stepletIds, pipelineId) {
        const grpMessage = 'Queuing Step';
        const cmdMessage = 'Info';
        const message = 'Skipping step because it is not eligible to execute.';
        await this.publishLogs(pipelineId, false, grpMessage, cmdMessage, message, stepletIds);
    }
    async publishSkipLogsWhenHookIsNotPosted(stepletIds, pipelineId) {
        const grpMessage = 'Queuing Step';
        const cmdMessage = 'Info';
        const message = 'Skipping the step because either Jenkins is down or our internal APIs are facing some issues. Please try rerunning the step after some time.';
        await this.publishLogs(pipelineId, false, grpMessage, cmdMessage, message, stepletIds);
    }
    async publishErrorLogsWhenHookIsNotAvailable(stepletIds, pipelineId) {
        const grpMessage = 'Queuing Step';
        const cmdMessage = 'Info';
        const message = `The step errored out because it doesn't contain a hook.`;
        await this.publishLogs(pipelineId, false, grpMessage, cmdMessage, message, stepletIds);
    }
    async publishErrorLogsWhenProjectIsInvalid(stepletIds, pipelineId) {
        const grpMessage = 'Queuing Step';
        const cmdMessage = 'Info';
        const message = 'The step errored out because the project information is invalid. Please check your configurations.';
        await this.publishLogs(pipelineId, false, grpMessage, cmdMessage, message, stepletIds);
    }
    async publishErrorLogsWhenProjectIsNull(stepletIds, pipelineId) {
        const grpMessage = 'Queuing Step';
        const cmdMessage = 'Info';
        const message = 'Step errored out because project is invalid. Please check your configurations.';
        await this.publishLogs(pipelineId, false, grpMessage, cmdMessage, message, stepletIds);
    }
    async publishErrorLogsWhenRtTokenIsNotGenerated(stepletIds, pipelineId) {
        const grpMessage = 'Queuing Step';
        const cmdMessage = 'Info';
        const message = `Failed because the step uses a local artifactory and couldn't generate an artifactory token for execution. This may occur due to insufficient or invalid artifactory permissions in the pipeline source. Please recheck the source configuration.`;
        await this.publishLogs(pipelineId, false, grpMessage, cmdMessage, message, stepletIds);
    }
    async publishErrorLogsWhenRunExecutionTokenIsNotUpdated(stepletIds, pipelineId) {
        const grpMessage = 'Queuing Step';
        const cmdMessage = 'Info';
        const message = 'Failed because the step cannot update runExecution token. Please try rerunning the step.';
        await this.publishLogs(pipelineId, false, grpMessage, cmdMessage, message, stepletIds);
    }
    async publishLicenseBreachedLogs(stepletIds, pipelineId) {
        const grpMessage = 'Queuing Step';
        const cmdMessage = 'Info';
        const message = `The system is currently operating at the maximum allowed number of concurrent builds. Please wait until one of the builds reaches the terminal state for this build to start.`;
        await this.publishLogs(pipelineId, false, grpMessage, cmdMessage, message, stepletIds);
    }
    async publishLogs(pipelineId, isSuccess, grp, cmd, messageString, stepletIds) {
        const grpConsoleId = (0, utility_1.uuidv4)();
        const cmdConsoleId = (0, utility_1.uuidv4)();
        const messageConsoleId = (0, utility_1.uuidv4)();
        const consoles = [
            {
                pipelineId: pipelineId,
                consoleId: grpConsoleId,
                parentConsoleId: 'root',
                type: shared_1.ConsoleTypeEnum.GRP,
                message: grp,
                timestamp: Date.now() * THOUSAND,
                timestampEndedAt: null,
                isSuccess: isSuccess,
                isShown: true,
                stage: shared_1.ConsoleStageEnum.SETUP
            },
            {
                pipelineId: pipelineId,
                consoleId: cmdConsoleId,
                parentConsoleId: grpConsoleId,
                type: shared_1.ConsoleTypeEnum.CMD,
                message: cmd,
                timestamp: Date.now() * THOUSAND,
                timestampEndedAt: null,
                isSuccess: isSuccess,
                isShown: true,
                stage: shared_1.ConsoleStageEnum.SETUP
            },
            {
                pipelineId: pipelineId,
                consoleId: messageConsoleId,
                parentConsoleId: cmdConsoleId,
                type: shared_1.ConsoleTypeEnum.MSG,
                message: messageString,
                timestamp: Date.now() * THOUSAND,
                timestampEndedAt: null,
                isSuccess: isSuccess,
                isShown: true,
                stage: shared_1.ConsoleStageEnum.SETUP
            }
        ];
        try {
            const publishStepLogPromises = [];
            for (const stepletId of stepletIds) {
                const message = {
                    payload: {
                        eventType: 'insertion',
                        payload: {
                            objectId: Number(stepletId),
                            objectType: 'steplet',
                            consoles: consoles
                        }
                    }
                };
                const publishStepLogPromise = this.logServiceQueueClient.publishMessage(message);
                publishStepLogPromises.push(publishStepLogPromise);
            }
            await Promise.all(publishStepLogPromises);
        }
        catch (error) {
            this.logger.warn(`Cannot publish consoles for stepletIds: ${(0, node_util_1.inspect)(stepletIds)} to core.logService due to error: ${(0, node_util_1.inspect)(error)}`);
        }
    }
}
exports.ConsoleLogRepository = ConsoleLogRepository;
//# sourceMappingURL=ConsoleLogRepository.js.map