"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StepRepository = void 0;
const node_util_1 = require("node:util");
const logger_1 = require("@jfrog-pipelines/logger");
const data_access_layer_1 = require("@jfrog-pipelines/data-access-layer");
const metadata_1 = require("@jfrog-pipelines/metadata");
const StepServiceDataAdapter_1 = require("../../../../driven/db/StepServiceDataAdapter");
const StepletDao_1 = require("../../../../driven/db/StepletDao");
const driven_1 = require("../../../../driven");
class StepRepository {
    constructor() {
        this.logger = logger_1.LoggerFactory.getLogger('StepRepository');
        this.stepServiceDataAdapter = new StepServiceDataAdapter_1.StepServiceDataAdapter();
        this.stepletDao = new StepletDao_1.StepletDao();
        this.stepDao = new driven_1.StepDao();
    }
    async getStepExecutionEligibility(stepId) {
        try {
            return await this.stepServiceDataAdapter.getStepExecutionEligibility(stepId);
        }
        catch (error) {
            this.logger.warn(`Cannot get executionEligibility for step with id: ${stepId} due to error: ${(0, node_util_1.inspect)(error)}`);
            return false;
        }
    }
    async requestStepApproval(stepId) {
        try {
            await this.stepServiceDataAdapter.approveSteps(stepId);
        }
        catch (error) {
            this.logger.warn(`Cannot request for approval with stepId: ${stepId} due to error: ${(0, node_util_1.inspect)(error)}`);
        }
    }
    async setWaitingForApproval(stepletId) {
        const query = new data_access_layer_1.SequelizeQueryBuilder({}).where('id', stepletId).attributes(['id', 'runtimePropertyBag']);
        const steplet = await this.stepletDao.findOne(query.build());
        if (steplet === null) {
            this.logger.warn('No steplet found for the id:', stepletId);
            return;
        }
        const update = {
            runtimePropertyBag: {
                ...steplet.runtimePropertyBag,
                waitingForApproval: new Date()
            }
        };
        await this.stepletDao.updateByInstance(steplet, update);
    }
    async setReadyAtAndMarkGroupInProgress(stepId) {
        try {
            const update = {
                groupInProgress: true,
                groupStartedAt: new Date(),
                readyAt: new Date()
            };
            const step = await this.stepDao.findById(stepId);
            await this.stepDao.updateByInstance(step, update);
            return true;
        }
        catch (error) {
            this.logger.warn(`Cannot update readyAt and mark groupInProgress with stepId: ${stepId} due to error: ${(0, node_util_1.inspect)(error)}`);
            return false;
        }
    }
    async postHook(hookId, message) {
        try {
            await this.stepServiceDataAdapter.postHook(hookId, message);
            return true;
        }
        catch (error) {
            this.logger.warn(`Cannot post hook with id: ${hookId} due to error: ${(0, node_util_1.inspect)(error)}`);
            return false;
        }
    }
    async skipSteps(steps) {
        const skipStepPromises = [];
        const stepIds = steps.map((step) => step.getId());
        const update = {
            statusCode: metadata_1.SKIPPED,
            pendingLogsComplete: true,
            endedAt: new Date()
        };
        for (const stepId of stepIds) {
            const skipStepPromise = this.stepServiceDataAdapter.putStepById(stepId, update);
            skipStepPromises.push(skipStepPromise);
        }
        try {
            await Promise.all(skipStepPromises);
        }
        catch (error) {
            this.logger.warn(`Cannot skip steps due to error: ${(0, node_util_1.inspect)(error)}`);
        }
    }
    async skipSteplets(steplets) {
        const skipStepletsPromises = [];
        const stepletIds = steplets.map((steplet) => steplet.getId());
        const update = {
            statusCode: metadata_1.SKIPPED,
            pendingLogsComplete: true,
            endedAt: new Date()
        };
        for (const stepletId of stepletIds) {
            const skipStepletPromise = this.stepServiceDataAdapter.putStepletById(stepletId, update);
            skipStepletsPromises.push(skipStepletPromise);
        }
        try {
            await Promise.all(skipStepletsPromises);
        }
        catch (error) {
            this.logger.warn(`Cannot skip steplets due to error: ${(0, node_util_1.inspect)(error)}`);
        }
    }
    async errorSteps(steps) {
        const errorStepPromises = [];
        const stepIds = steps.map((step) => step.getId());
        const update = {
            statusCode: metadata_1.ERROR,
            pendingLogsComplete: true,
            endedAt: new Date()
        };
        for (const stepId of stepIds) {
            const errorStepPromise = this.stepServiceDataAdapter.putStepById(stepId, update);
            errorStepPromises.push(errorStepPromise);
        }
        try {
            await Promise.all(errorStepPromises);
        }
        catch (error) {
            this.logger.warn(`Cannot skip steps due to error: ${(0, node_util_1.inspect)(error)}`);
        }
    }
    async errorSteplets(steplets) {
        const errorStepletsPromises = [];
        const stepletIds = steplets.map((steplet) => steplet.getId());
        const update = {
            statusCode: metadata_1.ERROR,
            pendingLogsComplete: true,
            endedAt: new Date()
        };
        for (const stepletId of stepletIds) {
            const errorStepletPromise = this.stepServiceDataAdapter.putStepletById(stepletId, update);
            errorStepletsPromises.push(errorStepletPromise);
        }
        try {
            await Promise.all(errorStepletsPromises);
        }
        catch (error) {
            this.logger.warn(`Cannot skip steplets due to error: ${(0, node_util_1.inspect)(error)}`);
        }
    }
}
exports.StepRepository = StepRepository;
//# sourceMappingURL=StepRepository.js.map