"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodePoolQueuer = void 0;
const node_util_1 = require("node:util");
const logger_1 = require("@jfrog-pipelines/logger");
const repository_1 = require("../repository");
const ConsoleLogRepository_1 = require("../repository/ConsoleLogRepository");
const BuildPlaneMessage_1 = require("../models/BuildPlaneMessage");
const DynamicPool_1 = require("./nodepool/DynamicPool");
const StaticPool_1 = require("./nodepool/StaticPool");
class NodePoolQueuer {
    constructor(queableStepAffinity) {
        this.queableStepAffinity = queableStepAffinity;
        this.logger = logger_1.LoggerFactory.getLogger(`${queableStepAffinity
            .getStepsToExecute()
            .map((step) => step.getId())
            .join(',')}:NodePoolQueuer`);
        this.nodePoolRepository = new repository_1.NodePoolRepository();
        this.stepRepository = new repository_1.StepRepository();
        this.consoleLogManager = new ConsoleLogRepository_1.ConsoleLogRepository();
        this.poolStrategy = queableStepAffinity.getNodePool()?.isDynamic()
            ? new DynamicPool_1.DynamicPool(queableStepAffinity.getNodePool())
            : new StaticPool_1.StaticPool(queableStepAffinity.getNodePool());
    }
    async queue() {
        try {
            const nodePool = this.queableStepAffinity.getNodePool();
            if (nodePool === null) {
                this.logger.error(`No nodepool found for the step affinity.`);
                return;
            }
            const nodePoolId = nodePool.getId();
            const stepsToExecute = this.queableStepAffinity.getStepsToExecute();
            if (stepsToExecute.length === 0) {
                this.logger.warn(`No steps found for runId: ${this.queableStepAffinity.getRunId()}, affinityGroup: ${this.queableStepAffinity.getAffinityGroup()}`);
                return;
            }
            const isGroupSkippable = await this.poolStrategy.checkNodeUnavailability();
            if (isGroupSkippable) {
                const steplets = this.queableStepAffinity.getSteplets();
                await this.skipGroup(stepsToExecute, steplets, this.queableStepAffinity.getPipelineId());
                return;
            }
            const isGroupQueable = await this.checkIfNodeIsAvailable(nodePool);
            if (!isGroupQueable) {
                this.logger.warn(`Cannot queue steps because nodepool: ${nodePool.getName()} has no available nodes`);
                return;
            }
            const buildPlaneMessage = await BuildPlaneMessage_1.BuildPlaneMessage.create(this.queableStepAffinity, nodePool, stepsToExecute);
            if (buildPlaneMessage === null) {
                this.logger.warn(`Cannot create buildPlane message`);
                return;
            }
            const isBuildPlaneMessageQueued = await buildPlaneMessage.queue();
            if (!isBuildPlaneMessageQueued) {
                this.logger.error(`BuildPlaneMessage was not queued`);
                return;
            }
            await this.nodePoolRepository.publishMessageToNodePoolService(nodePoolId);
            const stepletIds = this.queableStepAffinity.getSteplets().map((steplet) => steplet.getId());
            await this.consoleLogManager.publishNodePoolQueuerQueuedLogs(stepletIds, this.queableStepAffinity.getPipelineId(), nodePool.getName());
        }
        catch (error) {
            this.logger.warn(`Cannot queue steps due to error: ${(0, node_util_1.inspect)(error)}`);
        }
    }
    async checkIfNodeIsAvailable(nodePool) {
        const numberOfNodes = nodePool.getNodeThreshold();
        const buildPlaneMessagesCount = await this.nodePoolRepository.getBuildPlaneMessageCount(nodePool.getId());
        if (buildPlaneMessagesCount === null) {
            return false;
        }
        return numberOfNodes > buildPlaneMessagesCount;
    }
    async skipGroup(steps, steplets, pipelineId) {
        const stepIds = steps.map((step) => step.getId());
        await this.stepRepository.skipSteps(steps);
        await this.stepRepository.skipSteplets(steplets);
        await this.consoleLogManager.publishSkipLogsWhenNoNode(stepIds, pipelineId);
        return;
    }
}
exports.NodePoolQueuer = NodePoolQueuer;
//# sourceMappingURL=NodePoolQueuer.js.map