"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WaitingGroupManager = void 0;
const node_util_1 = require("node:util");
const utility_1 = require("@jfrog-pipelines/utility");
const logger_1 = require("@jfrog-pipelines/logger");
const modules_1 = require("../modules");
const acl_1 = require("../acl");
class WaitingGroupManager {
    constructor() {
        this.logger = logger_1.LoggerFactory.getLogger('WaitingGroupManager');
    }
    static rejectGroup(isOnDemandNodePool, nodes, waitingGroups, isEnforceNonRootNodes, isInitAsNonRoot, numberOfNodesNodepool) {
        if (isOnDemandNodePool && (0, utility_1.isEmpty)(numberOfNodesNodepool)) {
            this.rejectGroupByNonRootEnforcement(waitingGroups, isEnforceNonRootNodes, isInitAsNonRoot);
            return waitingGroups;
        }
        this.rejectGroupByNodes(isOnDemandNodePool, nodes, waitingGroups, numberOfNodesNodepool);
        this.rejectGroupByNonRootEnforcement(waitingGroups, isEnforceNonRootNodes, isInitAsNonRoot);
        return waitingGroups;
    }
    filterWaitingGroupsByApprovalRequired(waitingGroups) {
        const groupsRequiringApproval = [];
        const newWaitingGroups = [];
        const rejectedGroups = [];
        for (const waitingGroup of waitingGroups) {
            if (waitingGroup.requiresApproval && !waitingGroup.approvalRequestedAt) {
                groupsRequiringApproval.push(waitingGroup);
                continue;
            }
            newWaitingGroups.push(waitingGroup);
        }
        (0, utility_1.each)(groupsRequiringApproval, (group) => {
            if (!group.waitingForApproval) {
                group.rejectionContext.waitingForApproval = {
                    timestamp: new Date()
                };
                rejectedGroups.push(group);
            }
        });
        return {
            groupsRequiringApproval,
            waitingGroupsResult: newWaitingGroups,
            rejectedGroupsWaitingForApproval: rejectedGroups
        };
    }
    async setReadyAt(waitingGroups) {
        const dataAdapter = modules_1.DataAdapterFactory.getApiDataAdapter();
        const batchSize = 5;
        await (0, utility_1.eachLimit)(waitingGroups, batchSize, async (group) => {
            const isAlreadySet = this.checkIfGroupAlreadyHasReadyAtValue(group.groupStepIds);
            if (isAlreadySet) {
                return;
            }
            const firstStepsList = group.firstStepsList;
            if (firstStepsList.length === 0) {
                return;
            }
            const firstStepsListForUpdation = firstStepsList.map((firstStep) => {
                return Number.parseInt(firstStep, 10);
            });
            try {
                await dataAdapter.updateStepsReadyAt(firstStepsListForUpdation);
            }
            catch (error) {
                this.logger.warn(`Cannot update readyAt value for the group due to error: ${(0, node_util_1.inspect)(error)}`);
                throw new Error(`Cannot update readyAt value for the group due to error: ${(0, node_util_1.inspect)(error)}`);
            }
        });
    }
    async skippingByNodePoolPermissionsInAbsenceOfPipelineSource(pipelineSourceIds, waitingGroups, nodePool) {
        const syncRepository = new acl_1.SyncRepository();
        const query = `pipelineSourceIds=${pipelineSourceIds.join(',')}`;
        try {
            const pipelineSourceById = await syncRepository.getPipelineSourcesByIdByQuery(query);
            const missingPipelineSourceIds = this.getGroupsWithMissingPipelineSources(pipelineSourceById, waitingGroups, nodePool);
            if (!(0, utility_1.isEmpty)(missingPipelineSourceIds)) {
                this.logger.warn('PipelineSources cannot be found for the ids:', JSON.stringify(missingPipelineSourceIds));
                throw new Error(`PipelineSources cannot be found for the ids: ${JSON.stringify(missingPipelineSourceIds)}`);
            }
        }
        catch (error) {
            this.logger.warn('Cannot extract pipelineSourceBySourceId because of the error:', (0, node_util_1.inspect)(error));
            throw error;
        }
    }
    getGroupsWithMissingPipelineSources(pipelineSourceById, waitingGroups, nodePool) {
        const pipelineSourceIds = Object.keys(pipelineSourceById);
        const missingPipelineSourceIds = [];
        for (const group of waitingGroups) {
            const groupHasPipelineSource = pipelineSourceIds.includes(group.pipelineSourceId.toString());
            if (!groupHasPipelineSource) {
                missingPipelineSourceIds.push(group.pipelineSourceId);
            }
            else {
                this.updateSkipByNodePoolPermissions(nodePool, pipelineSourceById, group);
            }
        }
        return missingPipelineSourceIds;
    }
    updateSkipByNodePoolPermissions(nodePool, pipelineSourceById, group) {
        const nodePoolEnvironments = nodePool.environments;
        const pipelineSourceEnvironments = pipelineSourceById[group.pipelineSourceId].environments;
        const pipelineSourceEnvironmentsIntersectionNodePoolEnvironments = [];
        for (const environments of nodePoolEnvironments) {
            if (pipelineSourceEnvironments.includes(environments)) {
                pipelineSourceEnvironmentsIntersectionNodePoolEnvironments.push(environments);
            }
        }
        if ((0, utility_1.isEmpty)(pipelineSourceEnvironmentsIntersectionNodePoolEnvironments)) {
            group.skipByNodePoolPermissions = true;
        }
    }
    checkIfGroupAlreadyHasReadyAtValue(groupStepIds) {
        for (const group of groupStepIds) {
            if (!(0, utility_1.isEmpty)(group.readyAt)) {
                return true;
            }
        }
        return false;
    }
    static rejectGroupByNonRootEnforcement(waitingGroups, isEnforceNonRootNodes, isInitAsNonRoot) {
        const skipByNonRootEnforcement = this.setSkipByNonRootEnforcement(isEnforceNonRootNodes, isInitAsNonRoot);
        if (!skipByNonRootEnforcement) {
            return;
        }
        for (const group of waitingGroups) {
            group.skipByNonRootEnforcement = true;
        }
    }
    static rejectGroupByNodes(isOnDemandNodePool, nodes, waitingGroups, numberOfNodesNodepool) {
        const skipByNoNode = this.setSkipByNoNode(isOnDemandNodePool, nodes, numberOfNodesNodepool);
        if (!skipByNoNode) {
            return;
        }
        for (const group of waitingGroups) {
            group.skipByNoNode = true;
        }
    }
    static setSkipByNoNode(isOnDemandNodePool, nodes, numberOfNodesNodepool) {
        if (isOnDemandNodePool && !(0, utility_1.isEmpty)(numberOfNodesNodepool)) {
            return true;
        }
        if ((0, utility_1.isEmpty)(nodes)) {
            return true;
        }
        return false;
    }
    static setSkipByNonRootEnforcement(isEnforceNonRootNodes, isInitAsNonRoot) {
        if (isEnforceNonRootNodes && !isInitAsNonRoot) {
            return true;
        }
        return false;
    }
}
exports.WaitingGroupManager = WaitingGroupManager;
//# sourceMappingURL=WaitingGroupManager.js.map